@@ -1,13 +1,11 @@
-# Copyright 2009-2014 Luc Saffre
+# Copyright 2009-2016 Rumma & Ko Ltd
 # License: BSD (see file COPYING for details)
+"""Defines explicit code names for URL parameters
 
-#~ from django.db import models
-#~ from django.http import HttpResponse, Http404
-#~ from django.contrib.contenttypes.models import ContentType
-#~ from django.contrib.contenttypes import generic
-#~ from django.utils import simplejson as json
-#~ from django.conf import settings
-
+"""
+# import six
+# str = six.text_type
+from builtins import str
 
 _handle_attr_name = '_lino_ui_handle'
 
@@ -23,7 +21,10 @@ URL_PARAM_END_DATE = 'ed'
 URL_PARAM_START_DATE = 'sd'
 
 URL_PARAM_PARAM_VALUES = 'pv'
+"""Array of values of table parameters."""
+
 URL_PARAM_FIELD_VALUES = 'fv'
+"""Array of values of action parameters."""
 
 URL_PARAM_ACTION_NAME = 'an'
 
@@ -31,13 +32,12 @@ URL_PARAM_FORMAT = 'fmt'
 URL_PARAM_REQUESTING_PANEL = 'rp'
 
 URL_PARAM_MASTER_TYPE = 'mt'
-"""
-The pk of the ContentType of the master model.
+"""The content type of the master instance.
+
 """
 
 URL_PARAM_MASTER_PK = 'mk'
-"""
-The pk of the master instance.
+"""The primary key of the master instance.
 """
 
 URL_PARAM_USER_LANGUAGE = 'ul'
@@ -54,6 +54,14 @@ URL_PARAM_SUBST_USER = 'su'
 substitute user
 """
 
+URL_PARAM_LINO_VERSION = "lv"
+"""Version number of linoweb.js for version clash checking"""
+
+URL_PARAM_DEVICE_TYPE = 'dt'
+"""
+Connected device type (used by Openui5)
+"""
+
 URL_PARAM_KNOWN_VALUES = 'kv'
 """
 known values
@@ -65,6 +73,9 @@ URL_PARAM_ACTION_STEP = "as"
 URL_PARAM_GRIDFILTER = 'filter'
 URL_PARAM_FILTER = 'query'
 URL_PARAM_TAB = 'tab'
+"""The number of the active tab panel.
+"""
+
 #~ URL_PARAM_EXPAND = 'expand'
 #~ """
 #~ A string entered in the quick search field or in the text field of a combobox.
@@ -97,6 +108,7 @@ URL_PARAMS = [
     'URL_PARAM_REQUESTING_PANEL',
     'URL_PARAM_SHOW_PARAMS_PANEL',
     'URL_PARAM_SUBST_USER',
+    'URL_PARAM_LINO_VERSION',
     'URL_PARAM_USER_LANGUAGE',
     'URL_PARAM_ACTION_STEP',
     'URL_PARAM_SELECTED',
@@ -122,23 +134,36 @@ URL_FORMAT_HTML = 'html'
 DEFAULT_GC_NAME = 0
 
 
+ICON_NAMES = """
+arrow_join arrow_up arrow_down delete add book_link eye basket
+emoticon_smile pencil cross money
+application_form application_view_list application_view_detail
+disk hourglass date_add email_add email_go script script_add bell calendar
+printer lightning printer_delete arrow_divide
+page_white_acrobat page_excel
+html vcard vcard_add wrench transmit
+accept database_gear
+cancel flag_green date_next
+""".split()
+"""
+A list of all names allowed as
+:attr:`lino.core.actions.Action.icon_name`.
+"""
+
+# pdf --> page_white_acrobat
+# csv -> page_excel
+
+
 def dict2kw(d):
     newd = {}
-    for k, v in d.items():
+    for k, v in list(d.items()):
         newd[str(k)] = v
     return newd
 
 
-def authenticated_user(user):
-    #~ if user.is_anonymous():
-        #~ return None
-    return user
-
-
 def parse_boolean(v):
     if v in ('true', 'on', True):
         return True
     if v in ('false', 'off', False):
         return False
-    #~ raise Exception("Invalid boolean value %r for field %s" % (v,self.name))
     raise Exception("Invalid boolean value %r" % v)
