@@ -1,44 +1,30 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2009-2014 Luc Saffre
+# Copyright 2009-2020 Rumma & Ko Ltd
 # License: BSD (see file COPYING for details)
 
-"""Defines the classes `AbstractTable` and :class:`VirtualTable`.
+"""Defines the classes :class:`AbstractTable` and
+:class:`VirtualTable`.
 
 """
-
-import logging
-logger = logging.getLogger(__name__)
+import logging ; logger = logging.getLogger(__name__)
 
 import os
 import yaml
 
 from django.db import models
 from django.conf import settings
-from django.contrib.contenttypes.models import ContentType
-from django.contrib.contenttypes import generic
-# from django.core.exceptions import PermissionDenied
-
 from django.utils.translation import ugettext_lazy as _
 
 from lino.core import actors
 from lino.core import actions
 from lino.core import fields
-from lino.core import signals
-
-
-from .tablerequest import TableRequest
-
-from lino.core.utils import Handle
-
-from lino.utils.xmlgen.html import E
-# from lino.utils.appy_pod import PrintTableAction, PortraitPrintTableAction
+from lino.core.tablerequest import TableRequest
+from lino.core.utils import resolve_fields_list
 
 
 class InvalidRequest(Exception):
     pass
 
-from lino.utils.xmlgen.html import RstTable
-
 
 if False:  # 20130710
 
@@ -109,20 +95,21 @@ if False:  # 20130710
             f.write(yaml.dump(self.data))
 
 
-class TableHandle(Handle):
+class TableHandle(object):
     """
     For every table we create one "handle" per renderer.
+
+    See also :meth:`lino.core.actors.Actor.setup_handle` which is called during
+    startup and sets more attributes.
+
     """
 
     _layouts = None
 
     def __init__(self, actor):
         self.actor = actor
-        Handle.__init__(self)
-        #~ super(TableHandle,self).__init__()
 
     def __str__(self):
-        #~ return str(self.ui.__class__)+str(self.actor) + 'Handle'
         return str(self.actor) + 'Handle'
 
     def setup_layouts(self):
@@ -131,7 +118,7 @@ class TableHandle(Handle):
         self._layouts = [self.list_layout]
 
     def get_actor_url(self, *args, **kw):
-        return settings.SITE.ui.get_actor_url(self.actor, *args, **kw)
+        return settings.SITE.kernel.get_actor_url(self.actor, *args, **kw)
 
     def submit_elems(self):
         return []
@@ -146,8 +133,7 @@ class TableHandle(Handle):
         return lh.main.columns
 
     def get_slaves(self):
-        #~ return [ sl.get_handle(self.ui) for sl in self.actor._slaves ]
-        return [sl.get_handle(settings.SITE.ui) for sl in self.actor._slaves]
+        return [sl.get_handle() for sl in self.actor._slaves]
 
 
 class Group(object):
@@ -164,16 +150,17 @@ class Group(object):
 
 
 class AbstractTable(actors.Actor):
-
-    """An AbstractTable is the definition of a tabular data view,
-    usually displayed in a Grid (but it's up to the user
-    interface to decide how to implement this).
+    """An AbstractTable is the definition of a tabular data view, usually
+    displayed in a Grid (but it's up to the user interface to decide
+    how to implement this).
 
     Base class for :class:`Table <lino.core.dbtables.Table>` and
     :class:`VirtualTable <lino.core.tables.VirtualTable>`.
 
     """
 
+    abstract = True
+
     _handle_class = TableHandle
 
     hide_zero_rows = False
@@ -183,19 +170,58 @@ class AbstractTable(actors.Actor):
 
     """
 
-    column_names = '*'
+    column_names_m = None
+    """An optional alternative for :attr:`column_names` to use when
+    :attr:`mobile_view <lino.core.site.Site.mobile_view>` is True.
+
     """
-    A string that describes the list of columns of this table.
 
-    Default value is ``'*'``.
-
-    Lino will automatically create a :class:`dd.ListLayout` from this.
+    column_names = '*'
+    """A string that describes the list of columns of this table.
 
+    Lino will automatically create a
+    :class:`lino.core.layouts.ColumnsLayout` from this.
     This string must not contain any newline characters because a
-    ListLayout's `main` panel descriptor must be horizontal.
+    ColumnsLayout's `main` panel descriptor must be horizontal.
 
+    Default value is ``'*'``. Where all columns are included.
+    This wildcard character means "all columns which have not been
+    named explicitly can be selected by the user and inserted at
+    this point". It can be combined with explicitly specified names.
 
+    For example::
 
+      column_names = "name owner * date"
+
+    specifies that `name` and `owner` come first, followed by inserted
+    columns and finally by `date`.
+
+    If ``'*'`` is not present in the string only explicitly named
+    columns will be available.
+
+    See also :meth:`setup_column` and :meth:`get_column_names`.
+
+    """
+
+    tablet_columns = None
+    """
+    The columns that must remain visible when this table is rendered
+    on a tablet device.
+    """
+
+    mobile_columns = None
+    """
+    The columns that must remain visible when this table is rendered
+    on a mobile device.
+    """
+
+    popin_columns = None
+    """
+    The columns that must pop in below the first column if there is no
+    space to render them on the device.
+
+    If None: All columns not listed in mobile_columns nor Tablet_columns
+    will not pop-in.
     """
 
     start_at_bottom = False
@@ -203,12 +229,10 @@ class AbstractTable(actors.Actor):
     bottom*.  Unlike reverse ordering, the rows remain in their
     natural order, but when we open a grid on this table, we want it
     to start on the last page.
-    
-    First use case are :class:`ml.sales.InvoicesByJournal` and
-    :class:`ml.ledger.InvoicesByJournal`.
-    But the result is not yet satisfying.
 
-    New since :srcref:`docs/tickets/143`.
+    Use cases would be :class:`lino_xl.lib.sales.InvoicesByJournal` and
+    :class:`lino_xl.lib.ledger.InvoicesByJournal` but the result is not yet
+    satisfying.
 
     """
 
@@ -232,43 +256,53 @@ class AbstractTable(actors.Actor):
 
     get_data_rows = None
     """
-    Virtual tables *must* define this method, normal (model-based)
-    tables *may* define it.
+    Maybe deprecated.  Use get_request_queryset() instead.
+
+    Virtual tables *must* define this method, model-based tables *may* define
+    it. e.g. in case they need local filtering.
 
     This will be called with a
     :class:`lino.core.requests.TableRequest` object and is expected to
     return or yield the list of "rows"::
-    
+
         @classmethod
         def get_data_rows(self, ar):
             ...
             yield somerow
-            
-    Model tables may also define such a method in case they need local
-    filtering.
 
     """
 
     preview_limit = settings.SITE.preview_limit
-    """
-    The maximum number of rows to fetch when this table is being
+    """The maximum number of rows to fetch when this table is being
     displayed in "preview mode", i.e. (1) as a slave table in a detail
-    window or (2) as an item of the :xfile:`admin_main.html` returned
-    by :meth:`lino.core.site_def.Site.get_admin_main_items`.
+    window or (2) as a dashboard item (:meth:`get_dashboard_items
+    <lino.core.site.Site.get_dashboard_items>`) in
+    :xfile:`admin_main.html`.
 
     The default value for this is the :attr:`preview_limit
-    <ad.Site.preview_limit>` class attribute of your
-    :class:`Site <ad.Site>`, which itself has a hard-coded
+    <lino.core.site.Site.preview_limit>` class attribute of your
+    :class:`Site <lino.core.site.Site>`, which itself has a hard-coded
     default value of 15 and which you can override in your
     :xfile:`settings.py`.
-    
-    If you set this to `None`, preview requests for this table will
+
+    If you set this to `0`, preview requests for this table will
     request all rows.  Since preview tables usually have no paging
     toolbar, that's theoretically what we want (but can lead to waste
     of performance if there are many rows).
-    
-    Test case and description in :ref:`cosi.tested`.
-    
+
+    In React if set to `0` the paging toolbar which usually is
+    present in the detail view, will be removed, as it has no use, as
+    all rows wil be displayed.
+
+    Test case and description in the tested docs of :ref:`cosi`.
+
+
+
+    """
+
+    row_height = 1
+    """
+    Number of text rows per data row.
 
     """
 
@@ -279,7 +313,7 @@ class AbstractTable(actors.Actor):
 
     """
 
-    auto_fit_column_widths = False
+    auto_fit_column_widths = settings.SITE.auto_fit_column_widths
     """Set this to `True` if you want to have the column widths adjusted
     to always fill the available width.  This implies that there will
     be no horizontal scrollbar.
@@ -314,9 +348,9 @@ class AbstractTable(actors.Actor):
     **Value** : Application code should specify this as a *single
     string* containing a space-separated list of field names.  Lino
     will automatically resolve this during server startup using
-    :func:`dd.fields_list`.  The runtime value of this attribute is a
-    *set of strings*, each one the name of a data element. Defaults to
-    an empty set.
+    :func:`lino.core.utils.fields_list`.  The runtime value of this
+    attribute is a *set of strings*, each one the name of a data
+    element. Defaults to an empty set.
 
     **Inheritance** : Note that this can be specified either on a
     :class:`Model` or on a :class:`Table`.  Lino will make a union of
@@ -334,21 +368,20 @@ class AbstractTable(actors.Actor):
     """
 
     cell_edit = True
-    """
-    `True` to use ExtJS CellSelectionModel, `False` to use RowSelectionModel.
-    When True, the users cannot select multiple rows.
-    When False, the users cannot select and edit individual cells.
+    """`True` (default) to use ExtJS CellSelectionModel, `False` to use
+    RowSelectionModel.  When `True`, the users cannot select multiple
+    rows.  When `False`, the users cannot edit individual cells using
+    the :kbd:`F2` key..
 
     """
 
-    show_detail_navigator = False
+    show_detail_navigator = True
     """
     Whether a Detail view on a row of this table should have a navigator.
     """
 
     default_group = Group()
 
-    #~ default_action = GridEdit
     default_layout = 0
 
     typo_check = True
@@ -358,7 +391,7 @@ class AbstractTable(actors.Actor):
     Usually such a warning means that there is something wrong.
     """
 
-    slave_grid_format = 'grid'
+    display_mode = 'grid'
     """
     How to display this table when it is a slave in a detail
     window. Must be one of the following values:
@@ -367,14 +400,31 @@ class AbstractTable(actors.Actor):
     - `'summary'` to render a summary in a HtmlBoxPanel.
     - `'html'` to render plain html a HtmlBoxPanel.
 
-    Example: :class:`ml.households.SiblingsByPerson`.
+    See :doc:`/dev/table_summaries`.
 
     """
 
     stay_in_grid = False
+    """Set this to True if Lino should prefer grid mode and not open a
+    detail window on a newly created record.  :class:`SubmitDetail
+    <lino.core.actions.SubmitDetail>` closes the window when this is
+    True.
+
+    Usage example :class:`LanguageKnowledgesByPerson
+    <lino_xl.lib.cv.models.LanguageKnowledgesByPerson>`.
+
     """
-    Set this to True if Lino should not open a newly created record in
-    a detail window.
+
+    no_phantom_row = False
+    """Suppress a phantom row in situations where Lino would otherwise add
+    one.
+
+    Used for :class:`lino_xl.lib.ledger.ByJournal` where a phantom row
+    is disturbing.
+
+    TODO: Actually this option would not be necessary if the AJAX call
+    sent by a grid panel would include an option which says whether it
+    is main item or not.
 
     """
 
@@ -384,45 +434,96 @@ class AbstractTable(actors.Actor):
     """
 
     order_by = None
+    """If specified, this must be a tuple or list of field names which
+will be passed to Django's `order_by
+<https://docs.djangoproject.com/en/1.11/ref/models/querysets/#order-by>`__
+method in order to sort the rows of the queryset.
+
+    """
 
     filter = None
-    """
-    If specified, this must be a `models.Q` object (not a dict of
-    (fieldname -> value) pairs) which will be used as a filter.
+    """If specified, this must be a `models.Q` object (not a dict of
+    (fieldname -> value) pairs) which will be passed to Django's
+    `filter
+    <https://docs.djangoproject.com/en/1.11/ref/models/querysets/#filter>`__
+    method.
 
-    Unlike :attr:`known_values`, this can use the full range of
-    Django's `field lookup methods
-    <https://docs.djangoproject.com/en/dev/topics/db/queries/#field-lookups>`_
-
-    Note that if the user can create rows in a filtered table, you
-    should make sure that new records satisfy your filter condition by
-    default, otherwise you can get surprising behaviour if the user
+    Note that if you allow a user to insert rows into a filtered
+    table, you should make sure that new records satisfy your filter
+    condition, otherwise you can get surprising behaviour if the user
     creates a new row.
 
     If your filter consists of simple static values on some known
-    field, then you'll prefer to use :attr:`known_values` instead of
-    :attr:`filter.`
+    field, then you might prefer to use
+    :attr:`known_values  <lino.core.actors.Actor.known_values>`
+    instead because this will add automatic behaviour.
 
+    One advantage of :attr:`filter` over
+    :attr:`known_values  <lino.core.actors.Actor.known_values>`
+    is that this can use the full range of Django's `field lookup methods
+    <https://docs.djangoproject.com/en/1.11/topics/db/queries/#field-lookups>`_
 
     """
 
     exclude = None
+    """If specified, this must be dict which will be passed to Django's
+    `exclude
+    <https://docs.djangoproject.com/en/1.11/ref/models/querysets/#exclude>`__
+    method on the queryset.
+
+    """
 
     extra = None
     """
     Examples::
-    
+
       extra = dict(select=dict(lower_name='lower(name)'))
-      # (or if you prefer:) 
+      # (or if you prefer:)
       # extra = {'select':{'lower_name':'lower(name)'},'order_by'=['lower_name']}
-    
+
     List of SQL functions and which RDBMS supports them:
     http://en.wikibooks.org/wiki/SQL_Dialects_Reference/Functions_and_expressions/String_functions
-    
+
     """
 
-    def __init__(self, *args, **kw):
-        raise NotImplementedError("20120104")
+    hide_sums = False
+    """
+    Set this to True if you don't want Lino to display sums in a table
+    view.
+    """
+
+    use_paging = False
+    """
+    Set this to True in Extjs6 to not use a Buffered Store, and use a JsonStore with paging instead.
+    """
+
+    drag_drop_sequenced_field = None
+    """
+    Extjs6 only
+    Enables drag and drop reordering for a table.
+    Set to the field name that is used to track the order.
+    Only used in lino.mixins.sequenced.Sequenced. Field name seqno
+    """
+
+    focus_on_quick_search = False
+    """
+    If True , when the grid opens, the initial keyboard focus will be in the quick search field.
+    """
+
+    @classmethod
+    def class_init(cls):
+        super(AbstractTable, cls).class_init()
+        resolve_fields_list(cls, 'tablet_columns', set, {})
+        resolve_fields_list(cls, 'mobile_columns', set, {})
+        resolve_fields_list(cls, 'popin_columns', set, {})
+        if cls.model is not None:
+            if not issubclass(cls.model, models.Model):
+                if cls.model._lino_default_table is None:
+                    cls.model._lino_default_table = cls
+
+    # @classmethod
+    # def get_request_queryset(self, ar, **filter):
+    #     return []
 
     @classmethod
     def spawn(cls, suffix, **kw):
@@ -438,7 +539,7 @@ class AbstractTable(actors.Actor):
         If you override :meth:`parse_req`, then keep in mind that it will
         be called *before* Lino checks the requirements.  For example the
         user may be AnonymousUser even if the requirements won't let it be
-        executed.  `ar.subst_user.profile` may be None, e.g. when called
+        executed.  `ar.subst_user.user_type` may be None, e.g. when called
         from `find_appointment` in :class:`welfare.pcsw.Clients`.
 
         """
@@ -458,7 +559,7 @@ class AbstractTable(actors.Actor):
     def get_default_action(cls):
         #~ return actions.BoundAction(cls,cls.grid)
         #~ return 'grid'
-        return actions.GridEdit()
+        return actions.ShowTable()
 
     @classmethod
     def get_actor_editable(self):
@@ -476,7 +577,10 @@ class AbstractTable(actors.Actor):
         :attr:`column_names` which depends on the request.
 
         """
-        return self.column_names
+        if settings.SITE.mobile_view:
+            return self.column_names_m or self.column_names
+        else:
+            return self.column_names
 
     @classmethod
     def group_from_row(self, row):
@@ -508,33 +612,32 @@ class AbstractTable(actors.Actor):
         #~ f.close()
         #~ return "Grid Config has been saved to %s" % filename
 
-    @classmethod
-    def get_create_kw(self, ar, **kw):
-        """Deprecated.  This additional wrapper was used by
-        `lino.modlib.links` which anyway never worked.  We will soon
-        throw it away and call get_filter_kw directly instead.
-
-        """
-        return self.get_filter_kw(ar, **kw)
-
     @classmethod
     def get_filter_kw(self, ar, **kw):
         """
         Return a dict with the "master keywords" for this table
         and a given `master_instance`.
 
-        :class:`lino.modlib.tickets.models.EntriesBySession`
-        Blog Entries are not directly linked to a Session, but in the
-        Detail of a Session we want to display a table of related blog
+        For example, if you have two models :class:`Book` and
+        :class:`Author`, and a foreign key :attr:`Book.author` which
+        points to the author of the book, and a table `BooksByAuthor`
+        having `master_key` set to ``'author'``, then `get_filter_kw`
+        would return a dict `{'author': <PK>}` where PK is the primary
+        key of the action request's :attr:`master_instance
+        <lino.core.requests.BaseRequest.master_instance>`.
+
+        Another example is
+        :class:`lino_xl.lib.tickets.EntriesBySession`, where blog
+        entries are not directly linked to a session, but in the
+        detail of a session we want to display a table of related blog
         entries.
 
-        :class:`lino.modlib.households.models.SiblingsByPerson`
-        Household members are not directly linked to a Person, but
-        usually a Person is member of exactly one household, and in
-        the Detail of a Person we want to display the members of that
-        household.
-
+        :class:`lino_xl.lib.households.SiblingsByPerson` Household
+        members are not directly linked to a Person, but usually a
+        Person is member of exactly one household, and in the Detail
+        of a Person we want to display the members of that household.
         """
+        from lino.core.gfks import gfk2lookup, GenericForeignKey
         master_instance = ar.master_instance
         if self.master is None:
             pass
@@ -542,24 +645,9 @@ class AbstractTable(actors.Actor):
             # UsersWithClients as "slave" of the "table" Home
         elif self.master is models.Model:
             pass
-        elif isinstance(self.master_field, generic.GenericForeignKey):
-        #~ elif self.master is ContentType:
-            #~ print 20110415
-            if master_instance is None:
-                """
-                20120222 : here was only `pass`, and the two other lines
-                were uncommented. don't remember why I commented them out.
-                But it caused all tasks to appear in UploadsByController of
-                an insert window for uploads.
-                """
-                #~ pass
-                kw[self.master_field.ct_field] = None
-                kw[self.master_field.fk_field] = None
-            else:
-                ct = ContentType.objects.get_for_model(
-                    master_instance.__class__)
-                kw[self.master_field.ct_field] = ct
-                kw[self.master_field.fk_field] = master_instance.pk
+
+        elif isinstance(self.master_field, GenericForeignKey):
+            kw = gfk2lookup(self.master_field, master_instance, **kw)
         elif self.master_field is not None:
             if master_instance is None:
                 if not self.master_field.null:
@@ -580,7 +668,9 @@ class AbstractTable(actors.Actor):
                     # master_instance = None
                     return  # cannot add rows to this table
             kw[self.master_field.name] = master_instance
-
+        # else:
+        #     msg = "20150322 Cannot handle master {0}".format(master_instance)
+        #     raise Exception(msg)
         return kw
 
     #~ @classmethod
@@ -590,14 +680,19 @@ class AbstractTable(actors.Actor):
             #~ action = self.default_action
         #~ return TableRequest(ui,self,request,action,**kw)
 
+    # @fields.displayfield(_("Details"))
+    # def detail_pointer(cls, obj, ar):
+    #     # print("20181230 detail_pointer() {}".format(cls))
+    #     return obj.obj2href(ar)
+
     @classmethod
     def request(self, master_instance=None, **kw):
         """Return a new :class:`TableRequest
-        <lino.core.trequest.TableRequest>` on this table.
+        <lino.core.tablerequest.TableRequest>` on this table.
 
         If this is a slave table, the :attr:`master_instance
-        <lino.core.trequest.TableRequest.master_instance>` can be
-        specified as optional positional argument.
+        <lino.core.tablerequest.TableRequest.master_instance>` can be
+        specified as optional first positional argument.
 
         """
         kw.update(actor=self)
@@ -612,10 +707,7 @@ class AbstractTable(actors.Actor):
         To be combined with the `show` management command.
         """
         settings.SITE.startup()
-        #~ settings.SITE.ui
         if pk is not None:
-            #~ elem = self.get_row_by_pk(pk)
-            #~ elem = self.model.objects.get(pk=pk)
             if an is None:
                 an = self.default_elem_action_name
         elif an is None:
@@ -631,38 +723,18 @@ class AbstractTable(actors.Actor):
         kw = ar.response
         msg = kw.get('message')
         if msg:
-            print msg
-        url = kw.get('open_url') or kw.get('open_davlink_url')
+            print(msg)
+        url = kw.get('open_url') or kw.get('open_webdav_url')
         if url:
             os.startfile(url)
 
     @classmethod
-    def to_rst(cls, ar, column_names=None, header_level=None, **kwargs):
-        "Better name would be table2rst (analog to table2xhtml())"
-        fields, headers, widths = ar.get_field_info(column_names)
+    def add_quick_search_filter(cls, data, search_text):
+        """Add a filter to the given data iterator in order to apply a quick
+        search for the given `search_text`.
 
-        sums = [fld.zero for fld in fields]
-        rows = []
-        recno = 0
-        for row in ar.sliced_data_iterator:
-            recno += 1
-            rows.append([x for x in ar.row2text(fields, row, sums)])
-
-        if not cls.hide_sums:
-            has_sum = False
-            for i in sums:
-                if i:
-                    #~ print '20120914 zero?', repr(i)
-                    has_sum = True
-                    break
-            if has_sum:
-                rows.append([x for x in ar.sums2html(fields, sums)])
-
-        t = RstTable(headers, **kwargs)
-        s = t.to_rst(rows)
-        if header_level is not None:
-            s = E.tostring(E.h2(ar.get_title())) + s
-        return s
+        """
+        return data
 
 
 class VirtualTable(AbstractTable):
@@ -675,41 +747,88 @@ class VirtualTable(AbstractTable):
     Subclasses must define a :meth:`get_data_rows` method.
 
     """
-    pass
+    abstract = True
 
 
-class VentilatingTable(AbstractTable):
+class VentilatedColumns(VirtualTable):
     """
     A mixin for tables that have a series of automatically generated
     columns
     """
-
     ventilated_column_suffix = ':5'
 
-    @fields.virtualfield(models.CharField(_("Description"), max_length=30))
-    def description(self, obj, ar):
-        return unicode(obj)
+    column_names_template = ''
+    """
+    The template to use for :attr:`column_names`.   It should contain a string
+    ``{vcolumns}`` which will be replaced by a space-separated list of the column
+    names given by :meth:`get_ventilated_columns`.
+    """
+
+    abstract = True
 
     @classmethod
     def setup_columns(self):
-        self.column_names = 'description '
+        # if not "{vcolumns}" in self.column_names_template:
+        #     return
+        # self.column_names = 'description '
+        names = ''
         for i, vf in enumerate(self.get_ventilated_columns()):
             self.add_virtual_field('vc' + str(i), vf)
-            self.column_names += ' ' + vf.name + self.ventilated_column_suffix
+            names += ' ' + vf.name + self.ventilated_column_suffix
+
+        self.column_names = self.column_names_template.format(vcolumns=names)
+
         #~ logger.info("20131114 setup_columns() --> %s",self.column_names)
 
     @classmethod
     def get_ventilated_columns(self):
         return []
 
+class VentilatingTable(VentilatedColumns):
 
-from lino.core.signals import database_ready
+    abstract = True
+    column_names_template = 'description {vcolumns}'
+
+    @fields.virtualfield(models.CharField(_("Description"), max_length=30))
+    def description(self, obj, ar):
+        return str(obj)
 
 
-@signals.receiver(database_ready)
-def setup_ventilated_columns(sender, **kw):
-    if actors.actors_list is not None:
-        for a in actors.actors_list:
-            if issubclass(a, AbstractTable):
-                a.setup_columns()
-    settings.SITE.resolve_virtual_fields()
+class ButtonsTable(VirtualTable):
+    """
+
+    An abstract :class:`VirtualTable` with only one column and whose rows are
+    action buttons.
+
+    Subclasses must implement `get_data_rows` to yield action buttons.
+
+    Usage example
+    `lino_welfare.modlib.reception.models.FindDateByClientTable`.
+
+    """
+    abstract = True
+    column_names = 'button'
+    auto_fit_column_widths = True
+    window_size = (60, 20)
+    hide_top_toolbar = True
+
+    @fields.displayfield(_("Button"))
+    def button(self, obj, ar):
+        return obj
+
+
+# from lino.core.signals import post_analyze
+# from django.db.utils import DatabaseError
+
+# @signals.receiver(post_analyze)
+# def setup_ventilated_columns(sender, **kw):
+#     # print("20170308 SETUP_VENTILATED_COLUMNS")
+#     if actors.actors_list is not None:
+#         for a in actors.actors_list:
+#             if issubclass(a, AbstractTable) and not a.abstract:
+#                 try:
+#                     a.setup_columns()
+#                 except DatabaseError:
+#                     logger.debug(
+#                         "Ignoring DatabaseError in %s.setup_ventilated_columns", a)
+#     settings.SITE.resolve_virtual_fields()
