@@ -4,17 +4,17 @@
 .. autosummary::
    :toctree:
 
-    diag
+    install
+    prep
     initdb
-    initdb_demo
-    makedocs
-    makescreenshots
-    makeui
+    diag
+    dump2py
     mergedata
     monitor
+    run
     show
-    syncscreenshots
-
-
+    linod
+    resetsequences
+    makemigdump
 
 """
