@@ -1,12 +1,12 @@
+from builtins import str
 # -*- coding: UTF-8 -*-
-# Copyright 2011 Luc Saffre
+# Copyright 2011 Rumma & Ko Ltd
 # License: BSD (see file COPYING for details)
 
 import logging
 logger = logging.getLogger(__name__)
 
 import os
-import errno
 import sys
 from optparse import make_option
 from os.path import join
@@ -14,29 +14,24 @@ from os.path import join
 from django.db import models
 from django.conf import settings
 from django.utils.translation import ugettext as _
-from django.utils.encoding import force_unicode
+from django.utils.encoding import force_text
 from django.core.management import call_command
 from django.core.management.base import BaseCommand, CommandError
 from django.db.models import loading
-#~ from django.db import models
-#~ from django.contrib.contenttypes.models import ContentType
-#~ from lino.core import actors
-#~ from lino.utils import get_class_attr
 
 import lino
-from lino.core.dbutils import app_labels
-from lino.utils import confirm, curry
+#from lino.core.utils import app_labels
+from lino.utils import curry
 from lino.utils import rstgen
 from lino.utils.restify import doc2rst, abstract
-from lino.core import dbtables
+from lino.core import kernel
 
-#~ from lino.core.dbutils import makedirs_if_missing, full_model_name
-from lino.dd import full_model_name
+from lino.api.dd import full_model_name
 
 
 def fieldtype(f):
     if isinstance(f, models.ForeignKey):
-        return f.__class__.__name__ + " to " + refto(f.rel.to)
+        return f.__class__.__name__ + " to " + refto(f.remote_field.model)
     return f.__class__.__name__
 
 
@@ -73,11 +68,11 @@ def model_overview(model):
 
     def verbose_name(f):
         settings.SITE.set_language(None)
-        label_en = force_unicode(_(f.verbose_name))
+        label_en = force_text(_(f.verbose_name))
         babel_labels = []
         for lng in settings.SITE.languages[1:]:
             dbutils.set_language(lng.django_code)
-            label = force_unicode(_(f.verbose_name))
+            label = force_text(_(f.verbose_name))
             if label != label_en:
                 babel_labels.append(label)
         if babel_labels:
@@ -92,17 +87,17 @@ def model_overview(model):
         ]
         #~ for lng in babel.AVAILABLE_LANGUAGES:
             #~ babel.set_language(lng)
-            #~ cells.append(force_unicode(_(f.verbose_name)))
+            #~ cells.append(force_text(_(f.verbose_name)))
         #~ cells.append(f.help_text)
         return cells
     rows = [rowfmt(f) for f in model._meta.fields]
     s = rstgen.table(headers, rows)
 
-    model_reports = [r for r in dbtables.master_reports if r.model is model]
+    model_reports = [r for r in kernel.master_tables if r.model is model]
     if model_reports:
         s += '\n\nMaster tables: %s\n\n' % rptlist(model_reports)
     if getattr(model, '_lino_slaves', None):
-        s += '\n\nSlave tables: %s\n\n' % rptlist(model._lino_slaves.values())
+        s += '\n\nSlave tables: %s\n\n' % rptlist(list(model._lino_slaves.values()))
         #~ s += '\n\nSlave reports: '
         #~ s += ', '.join([name for name,rpt in model._lino_slaves.items()])
         #~ s += '\n\n'
@@ -112,7 +107,7 @@ def model_overview(model):
 def rptlist(l):
     return ', '.join([
         ":ref:`%s (%s) <%s>`" % (str(rpt),
-                                 force_unicode(rpt.label), report_ref(rpt))
+                                 force_text(rpt.label), report_ref(rpt))
         for rpt in l])
 
 
@@ -162,7 +157,7 @@ class GeneratingCommand(BaseCommand):
         logger.info("Generated %s files", self.generated_count)
 
     def generate(self, tplname, fn, **context):
-        from lino import rt
+        from lino.api import rt
         from Cheetah.Template import Template as CheetahTemplate
 
         #~ if self.tmpl_dir:
@@ -171,7 +166,7 @@ class GeneratingCommand(BaseCommand):
         tpl_filename = rt.find_config_file(tplname, self.tmpl_dir)
         if tpl_filename is None:
             raise Exception("No file %s found" % tplname)
-        if isinstance(tpl_filename, unicode):
+        if isinstance(tpl_filename, str):
             tpl_filename = tpl_filename.encode(sys.getfilesystemencoding())
         tpl_filename = os.path.abspath(tpl_filename)
         fn = join(self.output_dir, fn)
@@ -188,6 +183,9 @@ class GeneratingCommand(BaseCommand):
 
         logger.info("Generating %s", fn)
         #~ logger.info("Generating %s from %s",fn,tpl_filename)
+
+        def app_labels():
+            return [p.app_label for p in settings.SITE.installed_plugins]
         context.update(
             lino=lino,
             #~ models=models,
@@ -198,7 +196,7 @@ class GeneratingCommand(BaseCommand):
         #~ print 20110315, context
         tpl = CheetahTemplate(file=tpl_filename, namespaces=[context])
         #~ tpl = CheetahTemplate(file(tpl_filename).read(),namespaces=[context])
-        s = unicode(tpl)
+        s = str(tpl)
         #~ print s
         file(fn, 'w').write(s.encode('utf-8'))
         self.generated_count += 1
