@@ -1,9 +1,10 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2011 Luc Saffre
+# Copyright 2011 Rumma & Ko Ltd
 # License: BSD (see file COPYING for details)
 """
 Generate the local files for running :mod:`lino.ui.qx`.
 """
+from __future__ import print_function
 
 import logging
 logger = logging.getLogger(__name__)
@@ -22,25 +23,20 @@ from shutil import copytree, rmtree
 from django.db import models
 from django.conf import settings
 from django.utils.translation import ugettext as _
-from django.utils.encoding import force_unicode
+from django.utils.encoding import force_text
 from django.core.management import call_command
 from django.core.management.base import BaseCommand, CommandError
 from django.db.models import loading
-#~ from django.db import models
-#~ from django.contrib.contenttypes.models import ContentType
-#~ from lino.core import actors
-#~ from lino.utils import get_class_attr
 
 import lino
 from lino.core import dbtables
+from lino.core import kernel
 from lino.core import actions
-from lino.core.dbutils import app_labels
+from lino.core.utils import app_labels
 from lino.utils import confirm
 from lino.utils import rstgen
 from lino.core.menus import Menu, MenuItem
-#~ from lino.utils.jsgen import py2js
 from lino.utils import jsgen
-#~ from lino.core.dbutils import makedirs_if_missing
 from lino.management.commands.makedocs import GeneratingCommand
 
 
@@ -49,7 +45,7 @@ def a2class(a):
     return 'lino.%s_%s_%s' % (a.actor.app_label, a.actor._actor_name, a.name)
 
 
-#~ from django.utils.importlib import import_module
+#~ from importlib import import_module
 #~ m = import_module(settings.ROOT_URLCONF)
 #~ m.ui.make_local_files()
 
@@ -60,7 +56,7 @@ class Command(GeneratingCommand):
     def handle(self, *args, **options):
         #~ options.update(output_dir=QXAPP_PATH)
         if args:
-            print "Warning : ignored arguments", args
+            print("Warning : ignored arguments", args)
         QXAPP_PATH = os.path.join(
             settings.QOOXDOO_PATH, 'lino_apps', settings.SITE.project_name)
         args = [QXAPP_PATH]
@@ -99,19 +95,19 @@ class Command(GeneratingCommand):
         self.generate_class_file(
             'lino.Application', 'Application.js.tmpl', **context)
         #~ self.generate('Application',application_lines(self))
-        for rpt in (dbtables.master_reports
-                    + dbtables.slave_reports
-                    + dbtables.generic_slaves.values()):
-            rh = rpt.get_handle(ui)
+        for rpt in (kernel.master_tables
+                    + kernel.slave_tables
+                    + list(kernel.generic_slaves.values())):
+            rh = rpt.get_handle()
             #~ js += "Ext.namespace('Lino.%s')\n" % rpt
             #~ f.write("Ext.namespace('Lino.%s')\n" % rpt)
             context.update(rh=rh)
             for a in rpt.get_actions():
-                if isinstance(a, dbtables.GridEdit):
+                if isinstance(a, dbtables.ShowTable):
                     context.update(action=a)
                     self.generate_class_file(
                         a2class(a), 'XyzTableWindow.js.tmpl', **context)
-                if isinstance(a, actions.ShowDetailAction):
+                if isinstance(a, actions.ShowDetail):
                     context.update(action=a)
                     self.generate_class_file(a2class(a),
                                              'XyzDetailWindow.js.tmpl',
