@@ -1,16 +1,5 @@
 # -*- coding: UTF-8 -*-
-## Copyright 2014 Luc Saffre
-## This file is part of the Lino project.
-## Lino is free software; you can redistribute it and/or modify
-## it under the terms of the GNU Lesser General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or
-## (at your option) any later version.
-## Lino is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-## GNU Lesser General Public License for more details.
-## You should have received a copy of the GNU Lesser General Public License
-## along with Lino; if not, see <http://www.gnu.org/licenses/>.
+## Copyright 2014-2015 Rumma & Ko Ltd
 
 """
 Adds usage of the `AwesomeUploader
@@ -19,7 +8,7 @@ Rymarczyk, hosted at
 <https://code.google.com/p/awesomeuploader/source/checkout>.
 
 Example configuration in a :xfile:`settings.py` file or
-a :ref:`djangosite_local` module::
+a :xfile:`lino_local.py` file::
 
     ad.configure_plugin(
         'awesomeuploader',
@@ -39,6 +28,7 @@ from lino import ad
 
 
 class Plugin(ad.Plugin):
+    "See :doc:`/dev/plugins`."
 
     verbose_name = "Awesome Uploader"
 
@@ -66,3 +56,10 @@ class Plugin(ad.Plugin):
         yield self.build_media_url('swfupload.js')
         yield self.build_media_url('swfupload.swfobjectjs')
         yield self.build_media_url('AwesomeUploader.js')
+
+    def setup_main_menu(config, site, user_type, m):
+        mg = site.plugins.office
+        m = m.add_menu(mg.app_label, mg.verbose_name)
+        m.add_action('awesomeuploader.UploaderPanel')
+
+
