@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2011-2014 Luc Saffre
+# Copyright 2011-2014 Rumma & Ko Ltd
 # License: BSD (see file COPYING for details)
 
 """The :xfile:`models.py` module for the
@@ -14,7 +14,7 @@ logger = logging.getLogger(__name__)
 
 from django.conf import settings
 from django.utils.translation import ugettext_lazy as _
-from lino import dd, rt
+from lino.api import dd, rt
 
 
 class UploaderAction(dd.Action):
@@ -29,7 +29,6 @@ class UploaderAction(dd.Action):
 class UploaderPanel(dd.Frame):
 
     help_text = _("Open an window for multiple file uploads.")
-    # required = dd.required(user_groups='office')
     label = _("Uploader")
 
     @classmethod
@@ -37,16 +36,8 @@ class UploaderPanel(dd.Frame):
         return UploaderAction()
 
 
-system = dd.resolve_app('system')
-
-
-def setup_main_menu(site, ui, profile, m):
-    m = m.add_menu("office", system.OFFICE_MODULE_LABEL)
+def setup_quicklinks(site, user, m):
     m.add_action('awesomeuploader.UploaderPanel')
 
 
-def setup_quicklinks(site, ar, m):
-    m.add_action('awesomeuploader.UploaderPanel')
-
-
-__all__ = ['UploaderPanel']
+# __all__ = ['UploaderPanel']
