@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2011-2013 Luc Saffre
+# Copyright 2011-2013 Rumma & Ko Ltd
 # License: BSD (see file COPYING for details)
 """
 Adds all languages of the world to your database.
@@ -26,7 +26,7 @@ from django.conf import settings
 
 from lino.utils import dblogger as logger
 
-from lino import dd
+from lino.api import dd
 from lino.modlib.languages.models import Language
 
 
