@@ -1,4 +1,4 @@
-# Copyright 2009-2014 Luc Saffre
+# Copyright 2009-2016 Luc Saffre
 # License: BSD (see file COPYING for details)
 
 """This defines Lino default settings. You include this (directly or
@@ -8,15 +8,8 @@ indirectly) into your local :xfile:`settings.py` using::
 
 """
 
+from lino.api.ad import Site, configure_plugin, _
 
-import os
-import sys
-import datetime
-from tempfile import gettempdir
-from os.path import join, abspath, dirname, normpath
-import lino
-
-from lino.ad import Site, configure_plugin, _
 
 def TIM2LINO_LOCAL(alias, obj):
     """Hook for local special treatment on instances that have been
@@ -36,27 +29,26 @@ DEBUG = False
 TEMPLATE_DEBUG = DEBUG
 DEBUG_PROPAGATE_EXCEPTIONS = DEBUG
 
-
 ADMINS = [
     # ('Your Name', 'your_email@domain.com'),
 ]
 
 MANAGERS = ADMINS
 
-
 # Local time zone for this installation. Choices can be found here:
 # http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
 # although not all choices may be avilable on all operating systems.
 # If running in a Windows environment this must be set to the same as your
 # system time zone.
-#~ TIME_ZONE = 'Europe/Brussels'
-TIME_ZONE = None
+# TIME_ZONE = 'Europe/Brussels'
+# TIME_ZONE = None
+TIME_ZONE = 'UTC'  # 20190925 avoid "pytz.exceptions.UnknownTimeZoneError: None"
 # TIME_ZONE = 'America/Chicago'
 
 # Language code for this installation. All choices can be found here:
 # http://www.i18nguy.com/unicode/language-identifiers.html
-#~ LANGUAGE_CODE = 'de-BE'
-#~ LANGUAGE_CODE = 'fr-BE'
+# ~ LANGUAGE_CODE = 'de-BE'
+# ~ LANGUAGE_CODE = 'fr-BE'
 
 # ~ SITE_ID = 1 # see also fill.py
 
@@ -65,7 +57,7 @@ TIME_ZONE = None
 USE_I18N = True
 
 EMAIL_HOST = "mail.example.com"
-#EMAIL_PORT = ""
+# EMAIL_PORT = ""
 
 DATABASES = {
     'default': {
@@ -81,3 +73,12 @@ DATABASES = {
 
 SECRET_KEY = "20227"  # see :djangoticket:`20227`
 
+TEST_RUNNER = 'django.test.runner.DiscoverRunner'
+
+# disable migrations:
+# MIGRATION_MODULES = dict(contenttypes='lino.fake_migrations', sessions='lino.fake_migrations')
+
+
+
+# 20161114
+ALLOWED_HOSTS = ['testserver', 'localhost', '127.0.0.1', '::1']
