@@ -1,10 +1,38 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2013-2014 Luc Saffre
+# Copyright 2013-2018 Rumma & Ko Ltd
 # License: BSD (see file COPYING for details)
 
-"""A Sphinx extension used to write multilingual user documentation
+"""
+A Sphinx extension used to write multilingual user documentation
 for a Lino application.
 
+.. rst:directive:: lino2rst
+
+  Execute Python code and process the output as reStructuredText
+  source code.  This is like :rst:dir:`py2rst` but with the following
+  names defined:
+
+  :settings: The Django settings module which is active while building
+             the docs.
+            
+  :dd: The :mod:`lino.api.dd` module.
+            
+  :rt: The :mod:`lino.api.rt` module.
+            
+  Plus the app_label of each installed plugin.
+            
+
+.. rst:directive:: fields_list
+
+   Render a bullet list of the fields in a given model.  The first
+   word of the content is the model. If there are any other words,
+   then these are field names.  If no field names are specified, all
+   fields of the model are included.
+
+.. rst:role:: menupath
+
+   Render the given menu command specifier as a `menuselection` role.
+            
 .. rst:directive:: actor
 
 Usage::
@@ -21,18 +49,21 @@ that Actor or Model (data elements can be fields or actions)
 
 .. rst:role:: ddref
 
-Insert a reference to the named data dictionary item.
-The visible text will be automatically in the right language
-in multilingual userdocs.
-
+(Deprecated) Insert a reference to the named data dictionary item.
+The visible text will be automatically in the right language in
+multilingual userdocs.
 """
 
 from __future__ import unicode_literals, print_function
+from builtins import str
 
-from sphinx.util.compat import Directive
+from .base import menuselection_text
+
+from docutils.parsers.rst import Directive
+# from sphinx.util.compat import Directive
 
 
-from sphinx.roles import XRefRole
+from sphinx.roles import XRefRole, menusel_role
 from sphinx.util import ws_re
 from sphinx import addnodes
 
@@ -43,18 +74,21 @@ from django.db import models
 from django.conf import settings
 from django.utils.translation import ugettext_lazy as _
 from django.utils import translation
-from django.utils.encoding import force_unicode
+from django.utils.encoding import force_text
 
-from lino import dd, rt
+from lino.api import dd, rt
 
 from lino.core import actors
 from lino.core import actions
 from lino.core import choicelists
-from lino.core import dbtables
+from lino.core import kernel
 from atelier.utils import unindent
 from atelier import rstgen
-from lino.core.dbutils import full_model_name
+from lino.core.utils import full_model_name
 from lino.ad import Plugin
+from lino.utils.diag import analyzer
+from lino.core.actors import resolve_action
+from lino.modlib.users.choicelists import UserTypes
 
 from atelier.sphinxconf.insert_input import Py2rstDirective
 
@@ -75,13 +109,13 @@ def app_name(a):
 
 def actor_ref(rpt, text=None):
     if text is None:
-        text = force_unicode(rpt.label or rpt.title or str(rpt))
+        text = force_text(rpt.label or rpt.title or str(rpt))
     return ':ddref:`%s <%s>`' % (text, rpt)
 
 
 def model_ref(m, text=None):
     if text is None:
-        text = force_unicode(m._meta.verbose_name)
+        text = force_text(m._meta.verbose_name)
     return ':ref:`%s <%s>`' % (text, model_name(m))
 
 
@@ -97,8 +131,8 @@ def typeref(cls):
 
 def old_fieldtype(f):
     if isinstance(f, models.ForeignKey):
-        #~ return f.__class__.__name__ + " to " + refto(f.rel.to)
-        return f.__class__.__name__ + " to " + model_ref(f.rel.to)
+        #~ return f.__class__.__name__ + " to " + refto(f.rel.model)
+        return f.__class__.__name__ + " to " + model_ref(f.remote_field.model)
     return f.__class__.__name__
 
 
@@ -106,7 +140,7 @@ def fieldtype(f):
     s = typeref(f.__class__)
     if isinstance(f, models.ForeignKey):
         s = _("%(classref)s to %(model)s") % dict(
-            classref=s, model=model_ref(f.rel.to))
+            classref=s, model=model_ref(f.remote_field.model))
         #~ print(20130908, s)
     if isinstance(f, choicelists.ChoiceListField):
         s = _("%(classref)s to %(model)s") % dict(
@@ -118,10 +152,10 @@ def fields_ul(fields):
     helpless = []
 
     def field2li(fld):
-        s = "**%s**" % unicode(f.verbose_name).strip()
+        s = "**%s**" % str(f.verbose_name).strip()
         s += " (``%s``, %s)" % (f.name, fieldtype(f))
         if f.help_text:
-            s += " -- " + unicode(f.help_text)
+            s += " -- " + str(f.help_text)
             return s
         helpless.append(s)
         return None
@@ -159,7 +193,7 @@ def fields_table(fields):
         ]
         #~ for lng in babel.AVAILABLE_LANGUAGES:
             #~ babel.set_language(lng)
-            #~ cells.append(force_unicode(_(f.verbose_name)))
+            #~ cells.append(force_text(_(f.verbose_name)))
         #~ cells.append(f.help_text)
         return cells
     rows = [rowfmt(f) for f in fields if not hasattr(f, '_lino_babel_field')]
@@ -172,13 +206,13 @@ def get_actor_description(self):
     """
     body = "\n\n"
     if self.help_text:
-        body += unindent(force_unicode(self.help_text).strip()) + "\n\n"
+        body += unindent(force_text(self.help_text).strip()) + "\n\n"
 
     #~ ll = self.get_handle().list_layout
     #~ if ll is not None:
         #~ body += fields_table([ e.field for e in ll.main.columns] )
 
-    #~ model_reports = [r for r in dbtables.master_reports if r.model is self.model]
+    #~ model_reports = [r for r in kernel.master_tables if r.model is self.model]
     #~ if model_reports:
         #~ body += '\n\nMaster tables: %s\n\n' % rptlist(model_reports)
     #~ if getattr(model,'_lino_slaves',None):
@@ -193,64 +227,55 @@ def get_actor_description(self):
     #~ body = "\n\n"
     #~ help_text = getattr(self,'help_text',None)
     #~ if help_text:
-        #~ body += unindent(force_unicode(help_text).strip()) + "\n\n"
+        #~ body += unindent(force_text(help_text).strip()) + "\n\n"
 #~
     #~ body += fields_table(self._meta.fields)
     #~
     #~ return body
 
 
-IGNORED_ACTIONS = (actions.GridEdit, actions.SubmitDetail,
-                   actions.ShowDetailAction,
+IGNORED_ACTIONS = (actions.ShowTable, actions.SubmitDetail,
+                   actions.ShowDetail,
                    actions.DeleteSelected,
-                   actions.InsertRow, actions.SubmitInsert)
-
+                   actions.ShowInsert, actions.SubmitInsert)
 
 def menuselection(mi):
-    s = my_escape(unicode(mi.label).strip())
-    p = mi.parent
-    while p is not None:
-        if p.label:
-            s = my_escape(unicode(p.label).strip()) + " --> " + s
-        p = p.parent
-    return ":menuselection:`%s`" % s
+    return ":menuselection:`%s`" % menuselection_text(mi)
 
 
 def actions_ul(action_list):
     items = []
     for ba in action_list:
         label = ba.action.label
-        desc = "**%s** (" % unicode(label).strip()
+        desc = "**%s** (" % str(label).strip()
         if ba.action.action_name:
             desc += "``%s``" % ba.action.action_name
 
         desc += ", %s)" % typeref(ba.action.__class__)
         if ba.action.help_text:
-            desc += " -- " + unicode(ba.action.help_text)
+            desc += " -- " + str(ba.action.help_text)
         items.append(desc)
     return rstgen.ul(items)
 
-from lino.core.menus import find_menu_item
-
-
-def my_escape(s):
-    s = s.replace("\u25b6", "")
-    return s
+# from lino.core.menus import find_menu_item
 
 
 def actors_overview_ul(model_reports):
+    user_type = UserTypes.get_by_value('900')
+    # deprecated
     items = []
     for tb in model_reports:
         desc = actor_ref(tb)
-        #~ label = unicode(tb.title or tb.label)
+        #~ label = str(tb.title or tb.label)
         #~ desc += " (%s)" % str(tb)
         desc += " (%s)" % typeref(tb)
-        mi = find_menu_item(tb.default_action)
+        # mi = find_menu_item(tb.default_action)
+        mi = user_type.find_menu_item(tb.default_action)
         if mi is not None:
             desc += _(" (Menu %s)") % menuselection(mi)
             #~ print(unicode(mi.label).strip())
         if tb.help_text:
-            desc += " -- " + unicode(tb.help_text).strip()
+            desc += " -- " + str(tb.help_text).strip()
 
         items.append(desc)
     return rstgen.ul(items)
@@ -262,11 +287,11 @@ def resolve_name(name):
         return 1, settings.SITE.plugins.get(name)
         # return 1, dd.resolve_app(name)
     if len(l) == 3:
-        model = settings.SITE.modules.resolve(l[0] + '.' + l[1])
+        model = settings.SITE.models.resolve(l[0] + '.' + l[1])
         if model is None:
             raise Warning("Unkown name %s" % name)
         return 3, model.get_data_elem(l[2])
-    return len(l), settings.SITE.modules.resolve(name)
+    return len(l), settings.SITE.models.resolve(name)
 
 
 def form_lines():
@@ -309,21 +334,21 @@ class ddrefRole(XRefRole):
         lng = CurrentLanguage.get_current_value(env)
         with translation.override(lng):
             if isinstance(x, models.Field):
-                text = utils.unescape(unicode(x.verbose_name))
+                text = utils.unescape(str(x.verbose_name))
                 target = model_name(x.model) + '.' + x.name
                 # print(target)
             elif isinstance(x, Plugin):
-                text = utils.unescape(unicode(x.verbose_name))
+                text = utils.unescape(str(x.verbose_name))
                 target = settings.SITE.userdocs_prefix + target
 
             elif isinstance(x, type) and issubclass(x, models.Model):
-                text = utils.unescape(unicode(x._meta.verbose_name))
+                text = utils.unescape(str(x._meta.verbose_name))
                 target = model_name(x)
             elif isinstance(x, type) and issubclass(x, actors.Actor):
-                text = utils.unescape(unicode(x.title or x.label))
+                text = utils.unescape(str(x.title or x.label))
                 target = actor_name(x)
             elif isinstance(x, actions.Action):
-                text = utils.unescape(unicode(x.label))
+                text = utils.unescape(str(x.label))
                 target = actor_name(x)
             else:
                 raise Exception("Don't know how to handle %r" % x)
@@ -393,13 +418,13 @@ class CurrentProject(TempDataDirective):
 
 
 class Lino2rstDirective(Py2rstDirective):
-    """Defines the :rst:dir:`django2rst` directive."""
+    """Defines the :rst:dir:`lino2rst` directive."""
 
     def get_context(self):
         from django.conf import settings
         context = super(Lino2rstDirective, self).get_context()
         context.update(settings=settings)
-        context.update(settings.SITE.modules)
+        context.update(settings.SITE.models)
         context.update(dd=dd)
         context.update(rt=rt)
         return context
@@ -413,18 +438,45 @@ class Lino2rstDirective(Py2rstDirective):
 class ActorsOverviewDirective(Lino2rstDirective):
 
     def get_rst(self):
+        user_type = UserTypes.get_by_value('900')
         with translation.override(self.language):
             #~ set_language(lng)
             actor_names = ' '.join(self.content).split()
             items = []
             for an in actor_names:
-                cls = settings.SITE.modules.resolve(an)
+                cls = settings.SITE.models.resolve(an)
                 if not isinstance(cls, type):
                     raise Exception("%s is not an actor." % self.content[0])
-                items.append("%s : %s" % (actor_ref(cls), cls.help_text or ''))
+                desc = "**{0}** (:class:`{1} <{2}>`)".format(
+                    force_text(cls.label),
+                    cls.__name__,
+                    cls.__module__ + '.' + cls.__name__
+                )
+                mi = user_type.find_menu_item(cls.default_action)
+                if mi is not None:
+                    desc += _(" (Menu %s)") % menuselection(mi)
+                    #~ print(str(mi.label).strip())
+                if cls.help_text:
+                    desc += "  : " + force_text(cls.help_text).strip()
+
+                # items.append("%s : %s" % (actor_ref(cls), cls.help_text or ''))
+                items.append(desc)
             return rstgen.ul(items)
 
 
+class ShowFieldsDirective(Lino2rstDirective):
+
+    def get_rst(self):
+        with translation.override(self.language):
+            names = ' '.join(self.content).split()
+            if len(names) > 1:
+                field_names = ' '.join(names[1:])
+            else:
+                field_names = None
+            return analyzer.show_fields(
+                names[0], field_names=field_names)
+
+
 class FormDirective(Lino2rstDirective):
 
     def get_rst(self):
@@ -450,10 +502,10 @@ class ActorDirective(Lino2rstDirective):
                 fld = cls
                 s = ''
                 name = str(fld.model) + '.' + fld.name
-                title = force_unicode(fld.verbose_name).strip()
+                title = force_text(fld.verbose_name).strip()
                 
                 s += "\n.. index::\n   single: "
-                s += unicode(_('%(field)s (field in %(model)s)') % dict(
+                s += str(_('%(field)s (field in %(model)s)') % dict(
                     field=title, model=model_ref(fld.model)))
                 s += '\n\n'
                 s += rstgen.header(level, _("%s (field)") % title)
@@ -464,9 +516,9 @@ class ActorDirective(Lino2rstDirective):
 
             if isinstance(cls, Plugin):
                 s = ''
-                title = unicode(cls.verbose_name)
+                title = str(cls.verbose_name)
                 s += "\n.. index::\n   single: "
-                s += unicode(_('%s (app)') % title)
+                s += str(_('%s (app)') % title)
                 s += '\n\n.. _' + name + ':\n'
                 s += '\n'
                 s += rstgen.header(level, _("%s (app)") % title)
@@ -480,9 +532,9 @@ class ActorDirective(Lino2rstDirective):
 
                 s = ''
                 name = model_name(model).lower()
-                title = force_unicode(model._meta.verbose_name)
+                title = force_text(model._meta.verbose_name)
                 s += "\n.. index::\n   single: "
-                s += unicode(_('%(model)s (model in %(app)s)') % dict(
+                s += str(_('%(model)s (model in %(app)s)') % dict(
                     model=title, app=model._meta.app_label))
 
                 s += '\n\n'
@@ -502,8 +554,8 @@ class ActorDirective(Lino2rstDirective):
                     s += '\n\n'
 
                 model_reports = [
-                    r for r in dbtables.master_reports if r.model is cls]
-                model_reports += [r for r in dbtables.slave_reports
+                    r for r in kernel.master_tables if r.model is cls]
+                model_reports += [r for r in kernel.slave_tables
                                   if r.model is cls]
                 s += rstgen.boldheader(_("Views on %s") %
                                        cls._meta.verbose_name)
@@ -533,7 +585,7 @@ class ActorDirective(Lino2rstDirective):
 
             if issubclass(cls, actors.Actor):
 
-                title = force_unicode(cls.label or cls.title)
+                title = force_text(cls.label or cls.title)
                 indextext = _('%(actor)s (view in %(app)s)') % dict(
                     actor=title, app=cls.app_label)
                 name = actor_name(cls)
@@ -567,13 +619,27 @@ class ActorDirective(Lino2rstDirective):
         return [indexnode] + content
 
 
+def menupath_role(typ, rawtext, text, *args, **kwargs):
+    a = resolve_action(text)
+    user_type = UserTypes.get_by_value('900')
+    mi = user_type.find_menu_item(a)
+    if mi is None:
+        raise Exception("Unknown menu descriptor %s" % text)
+    text = menuselection_text(mi)
+    rawtext = menuselection(mi)
+    return menusel_role('menuselection', rawtext, text, *args, **kwargs)
+
+
 def setup(app):
 
     app.add_directive('form', FormDirective)
     app.add_directive('actor', ActorDirective)
     app.add_directive('actors_overview', ActorsOverviewDirective)
+    app.add_directive('fields_list', ShowFieldsDirective)
     app.add_role('ddref', ddrefRole())
+    app.add_role('menupath', menupath_role)
     app.add_directive('currentlanguage', CurrentLanguage)
     app.add_directive('currentproject', CurrentProject)
     app.add_directive('django2rst', Lino2rstDirective)  # backward compat
     app.add_directive('lino2rst', Lino2rstDirective)
+    
