@@ -1,148 +1,98 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2009-2014 Luc Saffre
+# Copyright 2009-2019 Rumma & Ko Ltd
 # License: BSD (see file COPYING for details)
 
 """:mod:`lino.utils` (the top-level module) contains a few often-used
 function for general use. It has also many subpackages and submodules.
 
-This is a tested document. To test it, run::
-
-  $ python setup.py test -s tests.DocsTests.test_utils
-
-
 .. autosummary::
    :toctree:
 
     addressable
     ajax
-    appy_pod
     choosers
     code
     config
+    dates
+    demonames
     daemoncommand
     dataserializer
     dbfreader
     dblogger
+    diag
+    djangotest
     dpy
-    editing
     html2odf
-    html2text
     html2xhtml
     mytidylib
-    htmlgen
     instantiator
+    jinja
     jscompressor
     jsgen
     latex
     log
     mdbtools
     media
-    memo
     mldbc
     mti
     odsreader
     pdf
+    pythontest
     pyuca
     quantities
     ranges
     requests
     restify
-    sendchanges
     screenshots
+    sendchanges
     sqllog
     ssin
     test
     textfields
     ucsv
+    report
 
 
-:func:`str2hex` and :func:`hex2str`
------------------------------------
-
->>> str2hex('-L')
-'2d4c'
-
->>> hex2str('2d4c')
-'-L'
-
->>> hex2str('')
-''
->>> str2hex('')
-''
-
-:func:`join_words`
-------------------
-
->>> print join_words('This','is','a','test')
-This is a test
-
->>> print join_words('This','is','','another','test')
-This is another test
-
->>> print join_words(None,None,None,'Third','test')
-Third test
-
 """
 
 #~ import logging
 #~ logger = logging.getLogger(__name__)
-
-
+from builtins import str
+from past.utils import old_div
 import sys
 import datetime
 import re
 from decimal import Decimal
+from collections import OrderedDict
+import dateparser
 
 # encapsulate where they come from:
 
-from atelier.utils import AttrDict, iif, ispure, assert_pure, confirm
+from atelier.utils import AttrDict, iif, ispure, assert_pure, confirm, isiterable, is_string, i2d, i2t
 from atelier import rstgen
-from atelier.utils import i2d
-from atelier.utils import i2t
+from etgen.utils import join_elems
+
 from lino.utils.cycler import Cycler
 from lino.utils.code import codefiles, codetime
 
-
-def isiterable(x):
-    "Returns `True` if the specified object is iterable."
-    try:
-        iter(x)
-    except TypeError:
-        return False
-    return True
-
-
 def join_words(*words):
     """
     Remove any empty item (None or ''), call unicode on each and
     join the remaining word using a single space.
-    """
-    return ' '.join([unicode(x) for x in words if x])
 
+    TODO: move this to etgen.html ?
 
-def join_elems(elems, sep=' '):
-    """
-    Examples:
-    >>> join_elems([1, 2, 3])
-    [1, ' ', 2, ' ', 3]
-    >>> join_elems([1, 2, 3],' / ')
-    [1, ' / ', 2, ' / ', 3]
-    >>> join_elems([])
-    []
-    """
-    if not callable(sep):
-        sep_value = sep
+    >>> print(join_words('This','is','a','test'))
+    This is a test
 
-        def sep():
-            return sep_value
-    l = []
-    s = None
-    for e in elems:
-        if s is not None:
-            l.append(s)
-        s = sep()
-        l.append(e)
-    return l
+    >>> print(join_words('This','is','','another','test'))
+    This is another test
+
+    >>> print(join_words(None, None, None, 'Third', 'test'))
+    Third test
+
+    """
+    return ' '.join([str(x) for x in words if x])
 
 
 def d2iso(d):
@@ -160,18 +110,6 @@ def get_class_attr(cl, name):
             return value
 
 
-def class_dict_items(cl, exclude=None):
-    if exclude is None:
-        exclude = set()
-    for k, v in cl.__dict__.items():
-        if not k in exclude:
-            yield k, v
-            exclude.add(k)
-    for b in cl.__bases__:
-        for k, v in class_dict_items(b, exclude):
-            yield k, v
-
-
 def call_optional_super(cls, self, name, *args, **kw):
     """
     Doesn't work. See `20110914`.
@@ -196,7 +134,7 @@ def call_on_bases(cls, name, *args, **kw):
         m = getattr(cls, name, None)
         # getattr will also return the classmethod defined on a base class,
         # which has already been called.
-        if m is not None and m.im_class is cls:
+        if m is not None and m.__self__.__class__ is cls:
             m(cls, *args, **kw)
 
     """Note: the following algorithm worked in Python 2.7 but not in 2.6,
@@ -213,7 +151,13 @@ def call_on_bases(cls, name, *args, **kw):
 
 
 def str2hex(s):
-    """Convert a string to its hexadecimal representation."""
+    """
+    Convert a string to its hexadecimal representation.
+
+    See examples in :func:`hex2str`.
+
+
+    """
     r = ''
     for c in s:
         r += hex(ord(c))[2:]
@@ -221,11 +165,29 @@ def str2hex(s):
 
 
 def hex2str(value):
-    """Convert the hexadecimal representation of a string to the original string."""
+    """
+    Convert the hexadecimal representation of a string to the original
+    string.
+
+    See also :func:`str2hex`.
+
+    >>> str2hex('-L')
+    '2d4c'
+
+    >>> hex2str('2d4c')
+    '-L'
+
+    >>> hex2str('')
+    ''
+    >>> str2hex('')
+    ''
+
+
+    """
     if len(value) % 2 != 0:
         raise Exception("hex2str got value %r" % value)
     r = ''
-    for i in range(len(value) / 2):
+    for i in range(old_div(len(value), 2)):
         s = value[i * 2:i * 2 + 2]
         h = int(s, 16)
         r += chr(h)
@@ -234,56 +196,67 @@ def hex2str(value):
 # http://snippets.dzone.com/posts/show/2375
 curry = lambda func, *args, **kw:\
     lambda *p, **n:\
-    func(*args + p, **dict(kw.items() + n.items()))
+    func(*args + p, **dict(list(kw.items()) + list(n.items())))
 
 
-class IncompleteDate:
+class IncompleteDate(object):
 
     """
     Naive representation of a potentially incomplete gregorian date.
 
-    Once upon a time in the year 2011:
-    >>> print IncompleteDate(2011, 0, 0).strftime("%d.%m.%Y")
+    For example you can say "Once upon a time in the year 2011":
+
+    >>> print(IncompleteDate(2011, 0, 0).strftime("%d.%m.%Y"))
     00.00.2011
 
-    >>> print IncompleteDate(1532, 0, 0)
+    Unlike :class:`datetime.date` objects an incomplete date can hold years
+    before 1970.
+
+    >>> print(IncompleteDate(1532, 0, 0))
     1532-00-00
-    >>> print IncompleteDate(1990, 0, 1)
-    1990-00-01
-    >>> print IncompleteDate(0, 6, 1)
+
+    On June 1st (but we don't say the year):
+
+    >>> print(IncompleteDate(0, 6, 1))
     0000-06-01
 
+    On the first day of the month in 1990:
+
+    >>> print(IncompleteDate(1990, 0, 1))
+    1990-00-01
+
     W.A. Mozart's birth date:
 
-    >>> print IncompleteDate(1756,1,27)
+    >>> print(IncompleteDate(1756, 1, 27))
     1756-01-27
 
     Christ's birth date:
 
-    >>> print IncompleteDate(-7,12,25)
+    >>> print(IncompleteDate(-7, 12, 25))
     -7-12-25
-    >>> print IncompleteDate(-7,12,25).strftime("%d.%m.%Y")
+    >>> print(IncompleteDate(-7, 12, 25).strftime("%d.%m.%Y"))
     25.12.-7
 
     Note that you cannot convert all incomplete dates
     to real datetime.date objects:
 
-    >>> IncompleteDate(-7,12,25).as_date()
+    >>> IncompleteDate(-7, 12, 25).as_date()
+    ... #doctest: +ELLIPSIS
     Traceback (most recent call last):
     ...
-    ValueError: year is out of range
+    ValueError: year...is out of range
 
-    >>> IncompleteDate(1756,1,27).as_date()
+    >>> IncompleteDate(1756, 1, 27).as_date()
     datetime.date(1756, 1, 27)
 
     An IncompleteDate is allowed to be complete:
 
     >>> d = IncompleteDate.parse('2011-11-19')
-    >>> print d
+    >>> print(d)
     2011-11-19
     >>> d.is_complete()
     True
-    >>> print repr(d.as_date())
+    >>> print(repr(d.as_date()))
     datetime.date(2011, 11, 19)
 
     >>> d = IncompleteDate.parse('2008-03-24')
@@ -298,6 +271,18 @@ class IncompleteDate:
     >>> d.get_age(i2d(20141025))
     6
 
+    Note that IncompleteDate can store invalid dates:
+
+    >>> d = IncompleteDate(2009, 2, 30)
+    >>> d.get_age(i2d(20160202))
+    6
+
+    >>> IncompleteDate(2009, 2, 32)
+    IncompleteDate('2009-02-32')
+
+    >>> IncompleteDate(2009, 32, 123)
+    IncompleteDate('2009-32-123')
+
     """
 
     def __init__(self, year, month, day):
@@ -305,12 +290,39 @@ class IncompleteDate:
 
     @classmethod
     def parse(cls, s):
+        """
+        Parse the given string and return an :class:`IncompleteDate`
+        object.
+
+        >>> IncompleteDate.parse('2008-03-24')
+        IncompleteDate('2008-03-24')
+
+        Belgian eID cards gave us some special challenges:
+
+        >>> IncompleteDate.parse('01.JUN. 1968')
+        IncompleteDate('1968-06-01')
+
+        >>> IncompleteDate.parse('JUN. 1968')
+        IncompleteDate('1968-06-00')
+        """
+
         if s.startswith('-'):
             bc = True
             s = s[1:]
         else:
             bc = False
-        y, m, d = map(int, s.split('-'))
+        try:
+            y, m, d = list(map(int, s.split('-')))
+        except ValueError:
+            pd = dateparser.parse(
+                s, settings={'STRICT_PARSING': True})
+            if pd is None:
+                pd = dateparser.parse(
+                    s, settings={'PREFER_DAY_OF_MONTH': 'first'})
+                y, m, d = pd.year, pd.month, 0
+            else:
+                y, m, d = pd.year, pd.month, pd.day
+            # raise Exception("Invalid date value {}".format(s))
         if bc:
             y = - y
         return cls(y, m, d)
@@ -343,25 +355,20 @@ class IncompleteDate:
         #~ s = fmt.replace("%Y",iif(self.bc,'-','')+str(self.year))
         if self.year == 0:
             s = fmt.replace("%Y", '0000')
-        else:
+        else:  # year might be negative
             s = fmt.replace("%Y", str(self.year))
         s = s.replace("%m", "%02d" % self.month)
         s = s.replace("%d", "%02d" % self.day)
         return s
 
-        #~ return self.strftime_(fmt,
-            #~ iif(self.bc,-1,1)*self.year,
-            #~ self.month,
-            #~ self.day)
-
     def as_date(self):
         return datetime.date(
-            #~ (self.year * iif(self.bc,-1,1)) or 1900,
             self.year or 1900,
-            self.month or 1,
-            self.day or 1)
+            self.month or 6,
+            self.day or 15)
 
     def get_age(self, today):
+        "Return age in years as integer."
         a = (self.month, self.day)
         b = (today.month, today.day)
         if a > b:
@@ -391,22 +398,22 @@ def moneyfmt(value, places=2, curr='', sep=',', dp='.',
     | trailneg: optional trailing minus indicator:  '-', ')', space or blank
 
     >>> d = Decimal('-1234567.8901')
-    >>> moneyfmt(d, curr='$')
-    '-$1,234,567.89'
-    >>> moneyfmt(d, places=0, sep='.', dp='', neg='', trailneg='-')
-    '1.234.568-'
-    >>> moneyfmt(d, curr='$', neg='(', trailneg=')')
-    '($1,234,567.89)'
-    >>> moneyfmt(Decimal(123456789), sep=' ')
-    '123 456 789.00'
-    >>> moneyfmt(Decimal('-0.02'), neg='<', trailneg='>')
-    '<0.02>'
+    >>> print(moneyfmt(d, curr='$'))
+    -$1,234,567.89
+    >>> print(moneyfmt(d, places=0, sep='.', dp='', neg='', trailneg='-'))
+    1.234.568-
+    >>> print(moneyfmt(d, curr='$', neg='(', trailneg=')'))
+    ($1,234,567.89)
+    >>> print(moneyfmt(Decimal(123456789), sep=' '))
+    123 456 789.00
+    >>> print(moneyfmt(Decimal('-0.02'), neg='<', trailneg='>'))
+    <0.02>
 
     """
     q = Decimal(10) ** -places      # 2 places --> '0.01'
     sign, digits, exp = value.quantize(q).as_tuple()
     result = []
-    digits = map(str, digits)
+    digits = list(map(str, digits))
     build, next = result.append, digits.pop
     if sign:
         build(trailneg)
@@ -435,12 +442,13 @@ def unicode_string(x):
     Because the message of an Exception may not be a unicode string.
 
     """
-    return unicode(x).encode(sys.getdefaultencoding(), 'backslashreplace')
+    return str(x).encode(sys.getdefaultencoding(), 'backslashreplace')
     # Python 2.6.6 said "Error in formatting: encode() takes no keyword arguments"
     #~ return unicode(x).encode(errors='backslashreplace')
 
 
 ONE_DAY = datetime.timedelta(days=1)
+ONE_WEEK = datetime.timedelta(days=7)
 
 
 def workdays(start, end):
@@ -466,7 +474,7 @@ def workdays(start, end):
     ...     a = i2d(start)
     ...     b = i2d(end)
     ...     if workdays(a,b) != expected:
-    ...        print "Got %d instead of %d for (%s,%s)" % (workdays(a,b),expected,a,b)
+    ...        print("Got %d instead of %d for (%s,%s)" % (workdays(a,b),expected,a,b))
 
     """
     #~ for d in range(start,end,ONE_DAY):
@@ -489,6 +497,7 @@ def camelize(s):
     'Abc def'
     >>> camelize("eID")
     'eID'
+
     """
     def f(k):
         if k.upper() != k:
@@ -518,6 +527,99 @@ def uncamel(s):
     return UNCAMEL_RE.sub(r'_\1', s).lower()
 
 
+def puts(s):
+    """A simplistic replacement for the `puts` function of `clint` which
+    has the problem of not supporting
+    `unicode strings <https://github.com/kennethreitz/clint/issues/48>`__.
+
+    This method is meant for issuing to the interactive console
+    messages which do not need to be logged because they just give
+    information about what's going on.
+
+    Currently this just prints the string to stdout using ``print``. I
+    prefer to use this over a plain ``print`` statement because I
+    guess that there will be problems (mainly thinking about the fact
+    that writing to stdout is considered an error in a wsgi
+    application).
+
+    """
+    # if isinstance(s, unicode):
+    #     print s.encode(locale.getpreferredencoding())
+    print(s)
+
+
+class SumCollector(object):
+    """A dictionary of sums to be collected using an arbitrary key.
+
+    Usage examples:
+
+    >>> sc = SumCollector()
+    >>> sc.collect("a", 12)
+    >>> sc.collect("a", None)
+    >>> sc.collect("a", 5)
+    >>> sc.a
+    17
+
+    >>> sc = SumCollector()
+    >>> sc.collect("a", 12)
+    >>> sc.collect("b", 23)
+    >>> sc.collect("a", 34)
+    >>> sc
+    OrderedDict([('a', 46), ('b', 23)])
+
+    >>> sc = SumCollector()
+    >>> from lino.utils.quantities import Duration
+    >>> sc.collect("a", Duration("0:30"))
+    >>> sc.collect("a", Duration("0:35"))
+    >>> sc.collect("b", Duration("0:00"))
+    >>> sc.a
+    Duration('1:05')
+    >>> sc.b
+    Duration('0:00')
+
+    This is also included in the default context used by the Jinja
+    renderer (:mod:`lino.modlib.jinja`) when rendering templates,
+    which makes it a more complete solution for a problem asked also
+    elsewhere, e.g. on `Stackoverflow
+    <http://stackoverflow.com/questions/7537439/how-to-increment-a-variable-on-a-for-loop-in-jinja-template>`__.
+
+    """
+    def __init__(self):
+        self._sums = OrderedDict()
+
+    def collect(self, k, value):
+        """Add the given value to the sum at the given key k."""
+        if value is None:
+            return
+        if k in self._sums:
+            self._sums[k] += value
+        else:
+            self._sums[k] = value
+
+    def __getattr__(self, k):
+        return self._sums.get(k)
+
+    def items(self, *args, **kwargs):
+        return self._sums.items(*args, **kwargs)
+
+    def __str__(self):
+        return str(self._sums)
+
+    def __repr__(self):
+        return repr(self._sums)
+
+
+class SimpleSingleton(object):
+    _instance = None
+
+    def __new__(cls, *args, **kwargs):
+        if not isinstance(cls._instance, cls):
+            cls._instance = object.__new__(cls, *args, **kwargs)
+        return cls._instance
+
+
+
+
 def _test():
     import doctest
     doctest.testmod()
