@@ -1,47 +1,27 @@
-# Copyright 2009-2014 Luc Saffre
+# Copyright 2009-2019 Rumma & Ko Ltd
 # License: BSD (see file COPYING for details)
 
 """Extends the possibilities for defining choices for fields of a
 Django model.
 
-- Context-sensitive choices
-- Non-limiting choices :
+- Context-sensitive choices (see :ref:`lino.dev.combo`)
+- Non-limiting choices (`force_selection` False) :
   specify a pick list of suggestions but leave the possibility
   to store manually entered values
 - :ref:`learning_combos`
 
-TODO: compare with `django-ajax-selects
-<https://github.com/crucialfelix/django-ajax-selects>`_
-
-
-.. _learning_combos:
-
-Learning Comboboxes
--------------------
-
-Choosers inspect the model, and if it defines a method
-`create_FOO_choice`, then the chooser will become "learning": the
-ComboBox will be told to accept also new values, and the server will
-handle these cases accordingly.
 
 """
+from builtins import object
+from builtins import str
 
-import logging
-logger = logging.getLogger(__name__)
-
-from django.db import models
-from django.contrib.contenttypes.models import ContentType
-from django.contrib.contenttypes.generic import GenericForeignKey
-
-from lino.utils.instantiator import make_converter
+from lino.api import rt
 from lino.core import constants
+from lino.core.utils import getrqdata
+from lino.utils.instantiator import make_converter
 
 
-def is_foreignkey(fld):
-    return isinstance(fld, (models.ForeignKey, GenericForeignKey))
-
-
-class BaseChooser:
+class BaseChooser(object):
     pass
 
 
@@ -59,9 +39,9 @@ class ChoicesChooser(FieldChooser):
 
 
 class Chooser(FieldChooser):
+    """A **chooser** holds information about the possible choices of a
+    field.
 
-    """
-    A Chooser holds information about the possible choices of a field.
     """
     #~ stored_name = None
     simple_values = False
@@ -75,7 +55,14 @@ class Chooser(FieldChooser):
         self.model = model
         #~ self.field = model._meta.get_field(fldname)
         self.meth = meth
-        if not is_foreignkey(field):
+        from lino.core.gfks import is_foreignkey
+        from lino.core.choicelists import ChoiceListField
+        if isinstance(field, ChoiceListField):
+            self.simple_values = getattr(meth, 'simple_values', False)
+            self.instance_values = getattr(meth, 'instance_values', True)
+            self.force_selection = getattr(
+                meth, 'force_selection', self.force_selection)
+        elif not is_foreignkey(field):
             self.simple_values = getattr(meth, 'simple_values', False)
             self.instance_values = getattr(meth, 'instance_values', False)
             self.force_selection = getattr(
@@ -89,7 +76,10 @@ class Chooser(FieldChooser):
         self.context_values = []
         self.context_fields = []
         for name in self.context_params:
+            if name == "ar":
+                continue
             f = self.get_data_elem(name)
+
             if f is None:
                 raise Exception(
                     "No data element '%s' in %s "
@@ -115,7 +105,6 @@ class Chooser(FieldChooser):
         if hasattr(model, "create_%s_choice" % field.name):
             self.can_create_choice = True
 
-        #~ if hasattr(model,"get_%s_display" % field.name):
         m = getattr(model, "%s_choice_display" % field.name, None)
         if m is not None:
             self.choice_display_method = m
@@ -140,18 +129,12 @@ class Chooser(FieldChooser):
             return self.model.get_param_elem(name)
         return de
 
-    #~ def get_data_elem(self,name):
-        #~ for vf in self.model._meta.virtual_fields:
-            #~ if vf.name == name:
-                #~ return vf
-        #~ return self.model._meta.get_field(name)
-
     def __call__(self, *args, **kw):
         for i, v in enumerate(args):
             kw[self.context_fields[i]] = v
         return self.get_choices(**kw)
 
-    def get_choices(self, **context):  # 20120918b
+    def get_choices(self, **context):
         """Return a list of choices for this chooser, using keyword parameters
         as context.
 
@@ -161,43 +144,42 @@ class Chooser(FieldChooser):
             args.append(context.get(varname, None))
         return self.meth(*args)
 
-    #~ def get_instance_choices(self,obj):
-        #~ "Return a list of choices for this chooser, using `obj` as context."
-        #~ args = []
-        #~ for varname in self.context_params:
-            #~ args.append(getattr(obj,varname,None))
-        #~ return self.meth(*args)
-
-    #~ def get_request_choices(self,ar,tbl):
-    def get_request_choices(self, request, tbl):
+    def get_request_choices(self, ar, tbl):
         """
-        Return a list of choices for this chooser, 
+        Return a list of choices for this chooser,
         using a HttpRequest to build the context.
         """
-        kw = {}
+        kw = {
+            "ar": ar,
+        }
+        # kw = {}
 
+        # ba = tbl.get_url_action(tbl.default_elem_action_name)
         # 20120202
         if tbl.master_field is not None:
-            mt = request.REQUEST.get(constants.URL_PARAM_MASTER_TYPE)
-            try:
-                master = ContentType.objects.get(pk=mt).model_class()
-            except ContentType.DoesNotExist, e:
-                pass
+            rqdata = getrqdata(ar.request)
+            if tbl.master is not None:
+                master = tbl.master
+            else:
+                mt = rqdata.get(constants.URL_PARAM_MASTER_TYPE)
+                ContentType = rt.models.contenttypes.ContentType
+                try:
+                    master = ContentType.objects.get(pk=mt).model_class()
+                except ContentType.DoesNotExist:
+                    master = None
 
-            pk = request.REQUEST.get(constants.URL_PARAM_MASTER_PK, None)
-            if pk:
+            pk = rqdata.get(constants.URL_PARAM_MASTER_PK, None)
+            if pk and master:
                 try:
                     kw[tbl.master_field.name] = master.objects.get(pk=pk)
-                except ValueError, e:
+                except ValueError:
                     raise Exception(
                         "Invalid primary key %r for %s", pk, master.__name__)
-                except master.DoesNotExist, e:
-                    # todo: ReportRequest should become a subclass of Dialog
-                    # and this exception should call dlg.error()
+                except master.DoesNotExist:
                     raise Exception("There's no %s with primary key %r" %
                                     (master.__name__, pk))
 
-        for k, v in request.GET.items():
+        for k, v in list(ar.request.GET.items()):
             kw[str(k)] = v
 
         # logger.info(
@@ -228,7 +210,7 @@ class Chooser(FieldChooser):
             return m(value)
         #~ raise NotImplementedError
         #~ assert not self.simple_values
-        m = getattr(obj, "get_" + self.field.name + "_display")
+        m = getattr(obj, "get_" + self.field.name + "_display", str)
         #~ if m is None:
             #~ raise Exception("")
         return m(value)
@@ -236,13 +218,18 @@ class Chooser(FieldChooser):
 
 
 def uses_simple_values(holder, fld):
-    "used by :class:`lino.ui.extjs.ext_store.Store`"
+    "used by :class:`lino.core.store`"
+    from lino.core.gfks import is_foreignkey
     if is_foreignkey(fld):
         return False
+    # if isinstance(fld, models.OneToOneRel):
+    #     return False
     if holder is not None:
         ch = holder.get_chooser_for_field(fld.name)
         if ch is not None:
             return ch.simple_values
+    if fld.choices is None:
+        return True
     choices = list(fld.choices)
     if len(choices) == 0:
         return True
@@ -255,12 +242,12 @@ def _chooser(make, **options):
     #~ options.setdefault('quick_insert_field',None)
     def chooser_decorator(fn):
         def wrapped(*args):
-            #~ print 20101220, args
             return fn(*args)
-        wrapped.context_params = fn.func_code.co_varnames[
-            1:fn.func_code.co_argcount]
-        #~ 20120918b wrapped.context_params = fn.func_code.co_varnames[2:fn.func_code.co_argcount]
-        for k, v in options.items():
+        cp = options.pop(
+            'context_params',
+            fn.__code__.co_varnames[1:fn.__code__.co_argcount])
+        wrapped.context_params = cp
+        for k, v in list(options.items()):
             setattr(wrapped, k, v)
         return make(wrapped)
         # return classmethod(wrapped)
