@@ -1,15 +1,13 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2012-2014 Luc Saffre
+# Copyright 2012-2017 Rumma & Ko Ltd
 # License: BSD (see file COPYING for details)
 
 """
-Defines some utilities to instpect the running Python code:
+Defines some utilities to inspect the running Python code.
 
-:func:`codefiles`
-:func:`codetime`
-and
-:func:`analyze_rst`
 """
+from builtins import str
+from builtins import object
 
 import logging
 logger = logging.getLogger(__name__)
@@ -18,18 +16,17 @@ import os
 import sys
 import time
 import fnmatch
-
-#~ def codefiles(pattern='.*', flags=0):
+from atelier import rstgen
 
 
 def codefiles(pattern='*'):
     """
-    Yield a list of the source files corresponding to the currently 
-    imported modules which match the given pattern
+    Yield a list of the source files corresponding to the currently
+    imported modules that match the given pattern
     """
     #~ exp = re.compile(pattern, flags)
 
-    for name, mod in sys.modules.items():
+    for name, mod in list(sys.modules.items()):
         #~ if name == 'lino.extjs' and pattern == '*':
             #~ logger.info("20130801 %r -> %r", name,mod.__file__)
         if fnmatch.fnmatch(name, pattern):
@@ -69,7 +66,9 @@ def codetime(*args, **kw):
 
 def is_start_of_docstring(line):
     for delim in '"""', "'''":
-        if line.startswith(delim) or line.startswith('u' + delim) or line.startswith('r' + delim) or line.startswith('ru' + delim):
+        if line.startswith(delim) or line.startswith('u' + delim) \
+           or line.startswith('r' + delim) \
+           or line.startswith('ru' + delim):
             return delim
 
 
@@ -122,9 +121,6 @@ class SourceFile(object):
         #~ self.count_code, count_total, count_blank, count_doc
 
 
-from lino.utils import rstgen
-
-
 def analyze_rst(*packages):
     """
     Example:
@@ -134,9 +130,13 @@ def analyze_rst(*packages):
       
     """
     fields = 'count_code count_doc count_comment count_total'.split()
-    headers = ["name"] + fields
+    headers = ["name", "code lines", "doc lines",
+               "comment lines", "total lines"]
     rows = []
 
+    def fmt(n):
+        return "{}k".format(round(n/1000.0, 1))
+
     total_sums = [0] * len(fields)
     for package in packages:
         sums = [0] * len(fields)
@@ -144,12 +144,13 @@ def analyze_rst(*packages):
             sf = SourceFile(name, filename)
             for i, k in enumerate(fields):
                 sums[i] += getattr(sf, k)
-        rows.append([package] + [str(n) for n in sums])
+        rows.append([package] + [fmt(n) for n in sums])
         for i, k in enumerate(fields):
             total_sums[i] += sums[i]
-    rows.append(['total'] + [str(n) for n in total_sums])
+    rows.append(['total'] + [fmt(n) for n in total_sums])
     return rstgen.table(headers, rows)
 
+
 if __name__ == "__main__":
     import doctest
     doctest.testmod()
