@@ -1,13 +1,13 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2009-2014 Luc Saffre
+# Copyright 2009-2017 Rumma & Ko Ltd
 # License: BSD (see file COPYING for details)
 
 """This defines the :class:`ConfigDirCache` which Lino instantiates
 and installs as :attr:`SITE.confdirs
-<lino.core.site_def.Site.confdirs>`.
+<lino.core.site.Site.confdirs>`.
 
 It creates a list `config_dirs` of all configuration directories by
-looping through :attr:`lino.core.site_def.Site.installed_plugins` and taking those
+looping through :attr:`lino.core.site.Site.installed_plugins` and taking those
 whose source directory has a :xfile:`config` subdir.
 
 The mechanism in this module emulates the behaviour of Django's and
@@ -20,6 +20,7 @@ One possibility might be to write a special Jinja Template class...
 """
 
 from __future__ import unicode_literals
+from builtins import object
 
 import logging
 logger = logging.getLogger(__name__)
@@ -29,6 +30,7 @@ from os.path import join, abspath, dirname, isdir
 import sys
 import codecs
 from fnmatch import fnmatch
+import six
 
 fs_encoding = sys.getfilesystemencoding() or sys.getdefaultencoding()
 
@@ -37,7 +39,7 @@ from lino.utils import iif
 SUBDIR_NAME = 'config'  # we might change this to "templates"
 
 
-class ConfigDir:
+class ConfigDir(object):
 
     """A directory that may contain configuration files.
 
@@ -57,36 +59,35 @@ class ConfigDirCache(object):
 
     def __init__(self, site):
         if self._init:
-            raise Exception("Oops")
+            raise Exception("Oops, ConfigDirCache was already instantiated!")
         self._init = True
         self.site = site
+        self.scan_config_dirs()
+        
+    def scan_config_dirs(self):
+        """Scan the file system and populate :attr:`config_dirs`."""
         config_dirs = []
 
-        for pth in site.get_settings_subdirs(SUBDIR_NAME):
-            config_dirs.append(ConfigDir(pth.decode(fs_encoding), False))
+        for pth in self.site.get_settings_subdirs(SUBDIR_NAME):
+            if six.PY2:
+                pth = pth.decode(fs_encoding)
+            config_dirs.append(ConfigDir(pth, False))
 
         def add_config_dir(name, mod):
             pth = join(dirname(mod.__file__), SUBDIR_NAME)
             if isdir(pth):
                 # logger.info("add_config_dir %s %s", name, pth)
-                config_dirs.append(ConfigDir(pth.decode(fs_encoding), False))
+                # config_dirs.append(ConfigDir(pth.decode(fs_encoding), False))
+                config_dirs.append(ConfigDir(pth, False))
 
-        # for p in site.installed_plugins:
-        #     add_config_dir(p.app_name, p.app_module.__file__)
-        site.for_each_app(add_config_dir)
+        self.site.for_each_app(add_config_dir)
 
         self.LOCAL_CONFIG_DIR = None
 
-        if site.is_local_project_dir:
-            p = join(site.project_dir, SUBDIR_NAME)
-            if isdir(p):
-                self.LOCAL_CONFIG_DIR = ConfigDir(p, True)
-                config_dirs.append(self.LOCAL_CONFIG_DIR)
-        #         print "20140625 Local config directory %s." % p
-        #     else:
-        #         print "20140625 No local config directory.", p
-        # else:
-        #     print "20140625 Not a local project directory."
+        p = self.site.cache_dir.child(SUBDIR_NAME)
+        if isdir(p):
+            self.LOCAL_CONFIG_DIR = ConfigDir(p, True)
+            config_dirs.append(self.LOCAL_CONFIG_DIR)
 
         config_dirs.reverse()
         self.config_dirs = tuple(config_dirs)
@@ -148,7 +149,7 @@ class ConfigDirCache(object):
         files = self.find_config_files('*' + template_ext, *groups)
         l = []
         template_ext
-        for name in files.keys():
+        for name in list(files.keys()):
             basename = name[:-len(template_ext)]
             chunks = basename.split('_')
             if len(chunks) > 1:
@@ -175,11 +176,11 @@ class ConfigDirCache(object):
         The `sort()` below must remove the filename extension (".dtl")
         because otherwise the frist Detail would come last.
         """
-        files = self.find_config_files(pattern, *groups).items()
+        files = list(self.find_config_files(pattern, *groups).items())
 
-        def fcmp(a, b):
-            return cmp(a[0][:-4], b[0][:-4])
-        files.sort(fcmp)
+        def fcmp(a):
+            return a[0][:-4]
+        files.sort(key=fcmp)
         for group in groups:
             prefix = group.replace("/", os.sep)
             for filename, cd in files:
@@ -227,7 +228,7 @@ def make_dummy_messages_file(src_fn, messages):
         return
     try:
         f = file(target_fn, 'w')
-    except IOError, e:
+    except IOError as e:
         logger.warning("Could not write file %s : %s", target_fn, e)
         return
     f.write("# this file is generated by Lino\n")
