@@ -1,43 +1,72 @@
-# Copyright 2011-2013 Luc Saffre
+# -*- coding: UTF-8 -*-
+# Copyright 2011-2016 Rumma & Ko Ltd
 # License: BSD (see file COPYING for details)
 
-"""
-  
-This module contains mainly a utility function :func:`html2odf` 
-which converts an ElementTree object 
-generated using :mod:`lino.utils.xmlgen.html`
-to a fragment of ODF.
+"""This module contains mainly a utility function :func:`html2odf`
+which converts an ElementTree object generated using
+:mod:`etgen.html` to a fragment of ODF.
 
->>> from lino.utils.xmlgen.html import E
+.. This is part of the Lino test suite. To test it individually, run:
+
+    $ python lino/utils/html2odf.py
+
+This is not trivial. The challenge is that HTML and ODF are quite
+different document representations. But something like this seems
+necessary. Lino uses it in order to generate .odt documents which
+contain (among other) chunks of html that have been entered using
+TinyMCE and stored in database fields.
+
+TODO: is there really no existing library for this task? I saw
+approaches which call libreoffice in headless mode to do the
+conversion, but this sounds inappropriate for our situation where we
+must glue together fragments from different sources. Also note that we
+use :mod:`appy.pod` to do the actual generation.
+
+Usage examples:
+
+>>> from etgen.html import E, tostring
 >>> def test(e):
-...     print E.tostring(e)
-...     print toxml(html2odf(e))
->>> test(E.p("This is a ",E.b("first")," test.")) #doctest: +NORMALIZE_WHITESPACE
+...     print (tostring(e))
+...     print (toxml(html2odf(e)))
+>>> test(E.p("This is a ", E.b("first"), " test."))
+... #doctest: +NORMALIZE_WHITESPACE
 <p>This is a <b>first</b> test.</p>
-<text:p xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0">This 
-is a <text:span text:style-name="Strong Emphasis">first</text:span> test.</text:p>
+<text:p xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0">This
+is a <text:span text:style-name="Strong Emphasis">first</text:span>
+test.</text:p>
 
->>> test(E.p(E.b("This")," is another test.")) #doctest: +NORMALIZE_WHITESPACE
+>>> test(E.p(E.b("This")," is another test."))
+... #doctest: +NORMALIZE_WHITESPACE
 <p><b>This</b> is another test.</p>
+<text:p xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0"><text:span
+text:style-name="Strong Emphasis">This</text:span> is another test.</text:p>
+
+>>> test(E.p(E.strong("This")," is another test."))
+... #doctest: +NORMALIZE_WHITESPACE
+<p><strong>This</strong> is another test.</p>
 <text:p xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0"><text:span 
 text:style-name="Strong Emphasis">This</text:span> is another test.</text:p>
 
->>> test(E.p(E.i("This")," is another test.")) #doctest: +NORMALIZE_WHITESPACE
+>>> test(E.p(E.i("This")," is another test."))
+... #doctest: +NORMALIZE_WHITESPACE
 <p><i>This</i> is another test.</p>
-<text:p xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0"><text:span 
+<text:p xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0"><text:span
 text:style-name="Emphasis">This</text:span> is another test.</text:p>
 
->>> test(E.td(E.p("This is another test."))) #doctest: +NORMALIZE_WHITESPACE
+>>> test(E.td(E.p("This is another test.")))
+... #doctest: +NORMALIZE_WHITESPACE
 <td><p>This is another test.</p></td>
-<text:p xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0">This 
+<text:p xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0">This
 is another test.</text:p>
 
->>> test(E.td(E.p(E.b("This")," is another test."))) #doctest: +NORMALIZE_WHITESPACE
+>>> test(E.td(E.p(E.b("This"), " is another test.")))
+... #doctest: +NORMALIZE_WHITESPACE
 <td><p><b>This</b> is another test.</p></td>
-<text:p xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0"><text:span 
+<text:p xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0"><text:span
 text:style-name="Strong Emphasis">This</text:span> is another test.</text:p>
 
->>> test(E.ul(E.li("First item"),E.li("Second item"))) #doctest: +NORMALIZE_WHITESPACE
+>>> test(E.ul(E.li("First item"),E.li("Second item")))
+... #doctest: +NORMALIZE_WHITESPACE
 <ul><li>First item</li><li>Second item</li></ul>
 <text:list xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0" 
 text:style-name="podBulletedList"><text:list-item><text:p 
@@ -45,65 +74,84 @@ text:style-name="podBulletItem">First item</text:p></text:list-item><text:list-i
 text:style-name="podBulletItem">Second item</text:p></text:list-item></text:list>
 
 N.B.: the above chunk is obviously not correct since Writer doesn't display it.
-(How can I debug a generated odt file? 
+(How can I debug a generated odt file?
 I mean if my content.xml is syntactically valid but Writer ...)
 Idea: validate it against the ODF specification using lxml
 
+Here is another HTML fragment which doesn't yield a valid result:
+
+>>> from lxml import etree
+>>> html = '<td><div><p><b>Bold</b></p></div></td>'
+>>> print(toxml(html2odf(etree.fromstring(html))))
+<text:p xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0"/>
 
 
-:func:`html2odf` converts bold text to a span with a 
-style named "Strong Emphasis". That's currently a hard-coded name, and the 
-caller must make sure that a style of that name is defined in the 
-document.
+
+
+
+:func:`html2odf` converts bold text to a span with a style named
+"Strong Emphasis". That's currently a hard-coded name, and the caller
+must make sure that a style of that name is defined in the document.
 
 The text formats `<i>` and `<em>` are converted to a style "Emphasis".
 
-
 Edge case:
 
->>> print toxml(html2odf("Plain string"))
+>>> print (toxml(html2odf("Plain string")))
 <text:p xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0">Plain string</text:p>
 
+>>> print (toxml(html2odf(u"Ein schöner Text")))
+<text:p xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0">Ein schöner Text</text:p>
+
+Not yet supported
+=================
+
+The following is an example for :ticket:`788`. Conversion fails if a
+sequence of paragraph-level items are grouped using a div:
+
+>>> test(E.div(E.p("Two numbered items:"),
+...    E.ol(E.li("first"), E.li("second"))))
+... #doctest: +NORMALIZE_WHITESPACE +IGNORE_EXCEPTION_DETAIL +ELLIPSIS
+Traceback (most recent call last):
+...
+IllegalText: The <text:section> element does not allow text
+
+
+>>> from lxml import etree
+>>> test(etree.fromstring('<ul type="disc"><li>First</li><li>Second</li></ul>'))
+<ul type="disc"><li>First</li><li>Second</li></ul>
+<text:list xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0" text:style-name="podBulletedList"><text:list-item><text:p text:style-name="podBulletItem">First</text:p></text:list-item><text:list-item><text:p text:style-name="podBulletItem">Second</text:p></text:list-item></text:list>
+
+
+>>> test(E.p(E.dl(E.dt("Foo"), E.dl("A foobar without bar."))))
+Traceback (most recent call last):
+...
+NotImplementedError: <dl> inside <text:p>
+
+
 """
 
+from __future__ import unicode_literals
+# from future import standard_library
+# standard_library.install_aliases()
+from builtins import str
+import six
 
 import logging
 logger = logging.getLogger(__name__)
 
-import os
-
-from lino.utils.xmlgen.html import E
-
-
-from django.utils.encoding import force_unicode
-from django.conf import settings
-
-
-from cStringIO import StringIO
-#~ from StringIO import StringIO
-
+from io import StringIO
+from lxml import etree
+from etgen.html import E, tostring
 
 def toxml(node):
+    """Convert an ODF node to a string with its XML representation."""
     buf = StringIO()
     node.toXml(0, buf)
     return buf.getvalue()
 
 
-from odf.opendocument import OpenDocumentText
-from odf.style import Style, TextProperties, ParagraphProperties
-from odf.style import TableColumnProperties, TableRowProperties, TableCellProperties
-#~ from odf.text import P
-from odf.element import Text
 from odf import text
-from odf.table import Table, TableColumns, TableColumn, TableHeaderRows, TableRows, TableRow, TableCell
-
-from cStringIO import StringIO
-
-
-def toxml(node):
-    buf = StringIO()
-    node.toXml(0, buf)
-    return buf.getvalue()
 
 
 #~ PTAGS = ('p','td','li')
@@ -112,9 +160,12 @@ PTAGS = ('p', 'td', 'div', 'table', 'tr')
 
 def html2odf(e, ct=None, **ctargs):
     """
-    Convert a :mod:`lino.utils.xmlgen.html` element to an ODF text element.
+    Convert a :mod:`etgen.html` element to an ODF text element.
     Most formats are not implemented.
     There's probably a better way to do this...
+
+    :ct: the root element ("container"). If not specified, we create one.
+
     """
     sections_counter = 1
     #~ print "20120613 html2odf()", e.tag, e.text
@@ -124,9 +175,9 @@ def html2odf(e, ct=None, **ctargs):
             #~ oe = text.P(**ctargs)
         #~ else:
             #~ oe = text.P(**ctargs)
-            #~ logger.info("20130201 %s",E.tostring(e))
+            #~ logger.info("20130201 %s", tostring(e))
             #~ raise NotImplementedError("<%s> without container" % e.tag)
-    if isinstance(e, basestring):
+    if isinstance(e, six.string_types):
         ct.addText(e)
         #~ oe = text.Span()
         #~ oe.addText(e)
@@ -140,7 +191,7 @@ def html2odf(e, ct=None, **ctargs):
 
     text_container = None
 
-    if e.tag == 'b':
+    if e.tag in ('b', 'strong'):
         #~ oe = text.Span(stylename='Bold Text')
         oe = text.Span(stylename='Strong Emphasis')
     elif e.tag == 'a':
@@ -185,7 +236,7 @@ def html2odf(e, ct=None, **ctargs):
         #~ else:
             #~ oe = text.P(**ctargs)
     else:
-        #~ logger.info("20130201 %s",E.tostring(e))
+        logger.info("20130201 %s", tostring(e))
         raise NotImplementedError("<%s> inside <%s>" % (e.tag, ct.tagName))
         #~ oe = text.Span()
 
