@@ -1,4 +1,4 @@
-# Copyright 2009-2014 Luc Saffre
+# Copyright 2009-2018 Rumma & Ko Ltd
 # License: BSD (see file COPYING for details)
 
 """Defines the :class:`Instantiator` class and some other utilities
@@ -6,20 +6,24 @@ used for generating database objects in :ref:`python fixtures <dpy>`.
 
 Example values:
 
+>>> import json
 >>> s = '<a href="javascript:Lino.pcsw.Clients.detail.run(\
 null,{ &quot;record_id&quot;: 116 })">BASTIAENSEN Laurent (116)</a>'
->>> GFK_HACK.match(s).groups()
-(u'pcsw.Clients', u'116')
+>>> print(json.dumps(GFK_HACK.match(s).groups()))
+["pcsw.Clients", "116"]
 
 >>> s = '<a href="javascript:Lino.cal.Guests.detail.run(\
 null,{ &quot;record_id&quot;: 6 })">Gast #6 ("Termin #51")</a>'
->>> GFK_HACK.match(s).groups()
-(u'cal.Guests', u'6')
+>>> print(json.dumps(GFK_HACK.match(s).groups()))
+["cal.Guests", "6"]
 
 """
 
 from __future__ import unicode_literals
 from __future__ import print_function
+from builtins import str
+import six
+from builtins import object
 
 
 import re
@@ -35,14 +39,12 @@ from dateutil import parser as dateparser
 
 from django.db import models
 from django.conf import settings
-from django.contrib.contenttypes.generic import GenericForeignKey
-# from django.contrib.contenttypes.models import ContentType
 
-from lino.core.dbutils import resolve_model, UnresolvedModel
+from lino.core.utils import resolve_model, UnresolvedModel
 
 from lino.utils import i2d  # for backward compatibility of .py fixtures
-from lino.core.dbutils import obj2str
-
+from lino.core.utils import obj2str
+from lino.core.fields import make_remote_field, RemoteField
 
 class DataError(Exception):
     pass
@@ -60,44 +62,44 @@ class Converter(object):
 class LookupConverter(Converter):
 
     """
-    A Converter for ForeignKey and ManyToManyField. 
+    A Converter for ForeignKey and ManyToManyField.
     If the lookup_field is a BabelField, then it tries all available languages.
     """
 
     def __init__(self, field, lookup_field):
         Converter.__init__(self, field)
-        model = field.rel.to
+        model = field.remote_field.model
         if lookup_field == 'pk':
             self.lookup_field = model._meta.pk
         else:
             self.lookup_field = model._meta.get_field(lookup_field)
-        #~ self.lookup_field = lookup_field
+        # self.lookup_field = lookup_field
 
     def lookup(self, value, **kw):
-        model = self.field.rel.to
+        model = self.field.remote_field.model
         if isinstance(value, model):
             return value
         return model.lookup_or_create(self.lookup_field, value, **kw)
 
-        #~ if isinstance(self.lookup_field,babel.BabelCharField):
-            #~ flt  = babel.lookup_filter(self.lookup_field.name,value,**kw)
-        #~ else:
-            #~ kw[self.lookup_field.name] = value
-            #~ flt = models.Q(**kw)
-        #~ try:
-            #~ return model.objects.get(flt)
-        #~ except MultipleObjectsReturned,e:
-            #~ raise model.MultipleObjectsReturned("%s.objects lookup(%r) : %s" % (model.__name__,value,e))
-        #~ except model.DoesNotExist,e:
-            #~ raise model.DoesNotExist("%s.objects lookup(%r) : %s" % (model.__name__,value,e))
+        # if isinstance(self.lookup_field,babel.BabelCharField):
+            # flt  = babel.lookup_filter(self.lookup_field.name,value,**kw)
+        # else:
+            # kw[self.lookup_field.name] = value
+            # flt = models.Q(**kw)
+        # try:
+            # return model.objects.get(flt)
+        # except MultipleObjectsReturned,e:
+            # raise model.MultipleObjectsReturned("%s.objects lookup(%r) : %s" % (model.__name__,value,e))
+        # except model.DoesNotExist,e:
+            # raise model.DoesNotExist("%s.objects lookup(%r) : %s" % (model.__name__,value,e))
 
 
 class DateConverter(Converter):
 
     def convert(self, **kw):
         value = kw.get(self.field.name)
-        if value is not None:
-            if not isinstance(value, datetime.date):
+        if not isinstance(value, datetime.date):
+            if value:  # keep out empty strings
                 if type(value) == int:
                     value = str(value)
                 d = dateparser.parse(value)
@@ -108,7 +110,11 @@ class DateConverter(Converter):
 
 class ChoiceConverter(Converter):
 
-    """Converter for :class:`ChoiceListField <lino.core.choicelists.ChoiceListField>`.
+    """Converter for :class:`ChoiceListField
+    <lino.core.choicelists.ChoiceListField>`.
+
+    If you specify a string, then it can be a *value* or a *name*.
+
     """
 
     def convert(self, **kw):
@@ -116,7 +122,11 @@ class ChoiceConverter(Converter):
 
         if value is not None:
             if not isinstance(value, self.field.choicelist.item_class):
-                kw[self.field.name] = self.field.choicelist.get_by_value(value)
+                # kw[self.field.name] = self.field.choicelist.get_by_value(value)
+                kw[self.field.name] = self.field.choicelist.to_python(value)
+                # if self.field.name == "vat_class":
+                #     print("20191210 convert {} from {} --> {}".format(
+                #         value, self.field.choicelist.items_dict, kw[self.field.name]))
         return kw
 
 
@@ -142,7 +152,7 @@ class ForeignKeyConverter(LookupConverter):
             else:
                 value = self.lookup(value)
             kw[self.field.name] = value
-            #~ logger.info("20111213 %s %s -> %r", self.field.name,self.__class__,value)
+            # logger.info("20111213 %s %s -> %r", self.field.name,self.__class__,value)
         return kw
 
 
@@ -158,7 +168,7 @@ class GenericForeignKeyConverter(Converter):
             else:
                 mo = GFK_HACK.match(value)
                 if mo is not None:
-                    actor = settings.SITE.modules.resolve(mo.group(1))
+                    actor = settings.SITE.models.resolve(mo.group(1))
                     pk = mo.group(2)
                     value = actor.get_row_by_pk(None, pk)
                     # ct = ContentType.objects.get_for_model(actor.model)
@@ -174,14 +184,14 @@ class ManyToManyConverter(LookupConverter):
     """Converter for ManyToMany fields."""
     splitsep = None
 
-    #~ def lookup(self,value):
-        #~ model = self.field.rel.to
-        #~ try:
-            #~ return model.objects.get(
-              #~ **{self.lookup_field: value})
-        #~ except model.DoesNotExist,e:
-            #~ raise DataError("%s.objects.get(%r) : %s" % (
-              #~ model.__name__,value,e))
+    # def lookup(self,value):
+        # model = self.field.remote_field.model
+        # try:
+            # return model.objects.get(
+              # **{self.lookup_field: value})
+        # except model.DoesNotExist,e:
+            # raise DataError("%s.objects.get(%r) : %s" % (
+              # model.__name__,value,e))
 
     def convert(self, **kw):
         values = kw.get(self.field.name)
@@ -194,12 +204,14 @@ class ManyToManyConverter(LookupConverter):
 
 
 def make_converter(f, lookup_fields={}):
+    from lino.core.gfks import GenericForeignKey
+
     if isinstance(f, models.ForeignKey):
         return ForeignKeyConverter(f, lookup_fields.get(f.name, "pk"))
     if isinstance(f, GenericForeignKey):
         return GenericForeignKeyConverter(f)
-    #~ if isinstance(f,fields.LinkedForeignKey):
-        #~ return LinkedForeignKeyConverter(f,lookup_fields.get(f.name,"pk"))
+    # if isinstance(f,fields.LinkedForeignKey):
+        # return LinkedForeignKeyConverter(f,lookup_fields.get(f.name,"pk"))
     if isinstance(f, models.ManyToManyField):
         return ManyToManyConverter(f, lookup_fields.get(f.name, "pk"))
     if isinstance(f, models.DateField):
@@ -208,12 +220,12 @@ def make_converter(f, lookup_fields={}):
         return DecimalConverter(f)
     from lino.core import choicelists
     if isinstance(f, choicelists.ChoiceListField):
-        #~ if f.name == 'p_book':
-            #~ print "20131012 b", f
+        # if f.name == 'p_book':
+            # print "20131012 b", f
         return ChoiceConverter(f)
 
 
-class Instantiator:
+class Instantiator(object):
     """A utility class to make python fixtures more compact. See
     :ref:`tutorial.instantiator`.
 
@@ -222,7 +234,7 @@ class Instantiator:
     """
 
     def __init__(self, model, fieldnames=None, converter_classes={}, **kw):
-        #~ self.model = resolve_model(model,strict=True)
+        # self.model = resolve_model(model,strict=True)
         self.model = resolve_model(model)
         if isinstance(self.model, UnresolvedModel):
             raise Exception("Instantiator on unresolved model %s", model)
@@ -231,10 +243,10 @@ class Instantiator:
                 pass
             self.build = noop
             return
-        #~ if self.model._meta.pk is None:
-            #~ raise Exception("Model %r is not installed (_meta.pk is None)." % self.model)
-        #~ if type(fieldnames) == str:
-        if isinstance(fieldnames, basestring):
+        # if self.model._meta.pk is None:
+            # raise Exception("Model %r is not installed (_meta.pk is None)." % self.model)
+        # if type(fieldnames) == str:
+        if isinstance(fieldnames, six.string_types):
             fieldnames = fieldnames.split()
         self.default_values = kw
         #self.fieldnames = fieldnames
@@ -249,8 +261,13 @@ class Instantiator:
                 if len(a) == 2:
                     name = a[0]
                     lookup_fields[name] = a[1]
-                field = self.model._meta.get_field(name)
-                self.fields.append(field)
+
+                rf = make_remote_field(model, name)
+                if rf:
+                    self.fields.append(rf)
+                else:
+                    field = self.model._meta.get_field(name)
+                    self.fields.append(field)
         # print " ".join(dir(model_class))
         # print " ".join(model_class._meta.fields)
         # for f in model_class._meta.fields:
@@ -264,8 +281,8 @@ class Instantiator:
                 cv = make_converter(f, lookup_fields)
             if cv is not None:
                 self.converters.append(cv)
-        #~ for f in model_class._meta.many_to_many:
-            #~ print "foo", f.name
+        # for f in model_class._meta.many_to_many:
+            # print "foo", f.name
 
     def is_active(self):
 
@@ -290,31 +307,47 @@ class Instantiator:
 
         """
         # logger.debug("Instantiator.build(%s,%r,%r)",self.model_class._meta.db_table,values,kw)
-        #~ i = 0
+        # i = 0
         kw['_m2m'] = {}
+        setters = []
         for i, v in enumerate(values):
-            if isinstance(v, basestring):
+            fld = self.fields[i]
+            if isinstance(v, six.string_types):
                 v = v.strip()
-                if len(v) > 0:
-                    kw[self.fields[i].name] = v
+                if len(v) == 0:
+                    continue
+            if isinstance(fld, RemoteField):
+                setters.append((fld, v))
             else:
-                kw[self.fields[i].name] = v
-            #~ i += 1
-        #~ kw.update(self.default_values)
+                kw[fld.name] = v
+
+        for name, v in kw.items():
+            if "__" in name:
+                del kw[name]
+                fld = make_remote_field(self.model, name)
+                setters.append((fld, v))
+
+        # kw.update(self.default_values)
         for k, v in self.default_values.items():
             kw.setdefault(k, v)
         for c in self.converters:
             kw = c.convert(**kw)
-        #~ if self.model.__name__ == 'Company':
-            #~ print 20130212, __file__, kw
-            #~ logger.info("20130212 field_cache for %s (%s)",self.model,
-              #~ ' '.join([f.name for f in self.model._meta._field_name_cache]))
+        # if self.model.__name__ == 'Company':
+            # print 20130212, __file__, kw
+            # logger.info("20130212 field_cache for %s (%s)",self.model,
+              # ' '.join([f.name for f in self.model._meta._field_name_cache]))
 
         m2m = kw.pop("_m2m")
+        # print("20191210 {}({})", self.model, kw)
         instance = self.model(**kw)
         instance.full_clean()
-        if m2m:
+        if m2m or len(setters):
             instance.save()
+
+        for fld, v in setters:
+            fld.setter(instance, v)
+
+        if m2m:
             for k, v in m2m.items():
                 queryset = getattr(instance, k)
                 queryset.add(*v)
@@ -323,7 +356,7 @@ class Instantiator:
 
 class InstanceGenerator(object):
     """
-    Usage example see :mod:`lino.modlib.humanlinks.fixtures`.
+    Usage example see :mod:`lino_xl.lib.humanlinks.fixtures`.
     """
     def __init__(self):
         self._objects = []
@@ -348,17 +381,41 @@ class InstanceGenerator(object):
         return rv
 
 
-def create_and_get(model, **kw):
+def create_row(model, **kw):
+    """Instantiate, full_clean, save and return a database object.
+
     """
-    Instantiate, full_clean, save 
-    and read back from database (the latter to avoid certain Django side effects)
-    """
-    model = resolve_model(model)
+    # model = resolve_model(model)
     o = model(**kw)
     o.full_clean()
     o.save()
-    return model.objects.get(pk=o.pk)
+    return o
 
+def create_or_update_row(model, lookup_values, new_values):
+    """
+    Update a single existing row that can be found using param: lookup values, with new_values.
+    If one doesn't exist, instantiates, full_cleans, saves and returns a database object
+    """
+    try:
+        o = model.objects.get(**lookup_values)
+        for attr, value in new_values.items():
+            setattr(o, attr, value)
+    except model.DoesNotExist: # If get receives more than one item will raise MultipleObjectsReturned:
+        o = model(**new_values)
+    o.full_clean()
+    o.save()
+    return o
+
+
+create = create_row  # backwards-compat
+
+def create_and_get(model, **kw):
+    """Create and then read back from database (the latter to avoid
+    certain Django side effects)
+
+    """
+    o = create(model, **kw)
+    return model.objects.get(pk=o.pk)
 
 def _test():
     import doctest
