@@ -1,38 +1,37 @@
-# Copyright 2009-2014 Luc Saffre
+# Copyright 2009-2016 Luc Saffre
 # License: BSD (see file COPYING for details)
 
 """
-A collection of tools around :doc:`/topics/mti`.
-See the source code of :mod:`lino.test_apps.mti`.
-Certainly not perfect, but works for me.
-I wrote it mainly to solve ticket :srcref:`docs/tickets/22`.
+A collection of tools around :doc:`multi-table inheritance </dev/mti>`.
 
 
 
 """
+from builtins import str
 import logging
 logger = logging.getLogger(__name__)
-import six
 
 from django.db import models
 from django.db import router
+# from django.db.models.deletion import Collector
 from django.db.models.deletion import Collector, DO_NOTHING
 from django.core.exceptions import ValidationError
+from django.utils.translation import ugettext_lazy as _
 
 
-from lino.core.dbutils import resolve_model
+from lino.core.utils import resolve_model
 from lino.core.fields import VirtualField
-from lino.core.signals import pre_remove_child, pre_add_child
+from lino.core.signals import pre_remove_child, pre_add_child  # , on_add_child
+from lino import AFTER17, DJANGO2
 
 
 class ChildCollector(Collector):
-
     """
     A Collector that does not delete the MTI parents.
     """
 
     def collect(self, objs, source=None, nullable=False, collect_related=True,
-        source_attr=None, reverse_dependency=False):
+                source_attr=None, reverse_dependency=False):
         # modified copy of django.db.models.deletion.Collector.collect()
         # changes:
         # DOES NOT Recursively collect concrete model's parent models.
@@ -49,63 +48,35 @@ class ChildCollector(Collector):
         model = new_objs[0].__class__
 
         if collect_related:
-            for related in model._meta.get_all_related_objects(
-                    include_hidden=True, include_proxy_eq=True,
-                    local_only=True):
+
+            related_objects = [
+                f for f in model._meta.get_fields(include_hidden=True)
+                if (f.one_to_many or f.one_to_one)
+                and f.auto_created and not f.concrete
+                and f.model is model  ## local_only
+            ]
+
+            # Before Django 1.10 we had:
+            # related_objects = model._meta.get_all_related_objects(
+            #         include_hidden=True, include_proxy_eq=True,
+            #         local_only=True)
+            
+            for related in related_objects:
                 field = related.field
-                if field.rel.on_delete == DO_NOTHING:
+                if field.remote_field.on_delete == DO_NOTHING:
                     continue
                 sub_objs = self.related_objects(related, new_objs)
                 if self.can_fast_delete(sub_objs, from_field=field):
                     self.fast_deletes.append(sub_objs)
                 elif sub_objs:
-                    field.rel.on_delete(self, field, sub_objs, self.using)
-            for field in model._meta.virtual_fields:
+                    field.remote_field.on_delete(self, field, sub_objs, self.using)
+            for field in model._meta.private_fields:
                 if hasattr(field, 'bulk_related_objects'):
                     # Its something like generic foreign key.
                     sub_objs = field.bulk_related_objects(new_objs, self.using)
                     self.collect(sub_objs,
                                  source=model,
-                                 source_attr=field.rel.related_name,
-                                 nullable=True)
-
-
-class OldChildCollector(Collector):
-    # version which worked for Django 1.5
-    def collect(self, objs, source=None, nullable=False, collect_related=True,
-                source_attr=None, collect_parents=True):
-        # modified copy from original Django code
-        new_objs = self.add(objs, source, nullable)
-        if not new_objs:
-            return
-        model = new_objs[0].__class__
-
-        if collect_related:
-            #~ for m,related in model._meta.get_all_related_objects_with_model(include_hidden=True):
-            for related in model._meta.get_all_related_objects(include_hidden=True, local_only=True):
-                field = related.field
-                if related.model._meta.auto_created:
-                    # Django 1.5.x:
-                    self.add_batch(related.model, field, new_objs)
-                else:
-                    sub_objs = self.related_objects(related, new_objs)
-                    #~ print 20130828, related.model._meta.concrete_model
-                    if not sub_objs:
-                        continue
-                    field.rel.on_delete(self, field, sub_objs, self.using)
-
-            # TODO This entire block is only needed as a special case
-            # to support cascade-deletes for GenericRelation. It
-            # should be removed/fixed when the ORM gains a proper
-            # abstraction for virtual or composite fields, and GFKs
-            # are reworked to fit into that.
-            for relation in model._meta.many_to_many:
-                if not relation.rel.through:
-                    sub_objs = relation.bulk_related_objects(
-                        new_objs, self.using)
-                    self.collect(sub_objs,
-                                 source=model,
-                                 source_attr=relation.rel.related_name,
+                                 source_attr=field.remote_field.related_name,
                                  nullable=True)
 
 
@@ -122,19 +93,29 @@ def get_child(obj, child_model):
 
 def delete_child(obj, child_model, ar=None, using=None):
     """
-    Delete the `child_model` instance related to `obj` without 
+    Delete the `child_model` instance related to `obj` without
     deleting the parent `obj` itself.
     """
-    #~ logger.info(u"delete_child %s from %s",child_model.__name__,obj)
+    # logger.info(u"delete_child %s from %s",child_model.__name__,obj)
     using = using or router.db_for_write(obj.__class__, instance=obj)
     child = get_child(obj, child_model)
     if child is None:
         raise Exception("%s has no child in %s" % (obj, child_model.__name__))
-    #~ msg = child_model._lino_ddh.disable_delete(child)
-    msg = child.disable_delete(ar)
+
+    # msg = child.disable_delete(ar)
+    ignore_models = set()
+    # for m in models_by_base(obj.__class__):
+    #     ignore_models.remove(child_model)
+    msg = child._lino_ddh.disable_delete_on_object(
+        obj, ignore_models)
     if msg:
         raise ValidationError(msg)
-    #~ logger.debug(u"Delete child %s from %s",child_model.__name__,obj)
+    # logger.debug(u"Delete child %s from %s",child_model.__name__,obj)
+
+    # 20160720 TODO: Django has added the keep_parents argument, and
+    # we should use this before Django 1.10 but this still seems to
+    # delete objects that are related to parents. So we sill cannot
+    # use it.
     if True:
         collector = ChildCollector(using=using)
         collector.collect([child])
@@ -148,13 +129,13 @@ def delete_child(obj, child_model, ar=None, using=None):
         # concrete_model = child_model._meta.concrete_model
         # for ptr in six.itervalues(concrete_model._meta.parents):
         #     if ptr:
-        #         # raise Exception(repr(ptr.rel.to))
-        #         del collector.data[ptr.rel.to]
+        #         # raise Exception(repr(ptr.rel.model))
+        #         del collector.data[ptr.rel.model]
 
     else:
-        collector = ChildCollector(using=using)
+        collector = Collector(using=using)
         collector.collect([child], source=obj.__class__,
-                          nullable=True, collect_parents=False)
+                          nullable=True, keep_parents=True)
     collector.delete()
 
     #~ setattr(obj,child_model.__name__.lower(),None)
@@ -168,9 +149,13 @@ def delete_child(obj, child_model, ar=None, using=None):
     # field and saves the form a second time.
 
 
-def insert_child(obj, child_model, **attrs):
-    """
-    Create and save an instance of `child_model` from existing `obj`.
+def insert_child(obj, child_model, full_clean=False, **attrs):
+    """Create and save an instance of `child_model` from existing `obj`.
+
+    If `full_clean` is True, call full_clean on the newly created
+    object. Default is `False` because this was the historic
+    behaviour.
+
     """
     #~ assert child_model != obj.__class__
     #~ if child_model == obj.__class__:
@@ -179,22 +164,50 @@ def insert_child(obj, child_model, **attrs):
             #~ obj.__class__.__name__)
     parent_link_field = child_model._meta.parents.get(obj.__class__, None)
     if parent_link_field is None:
-        raise ValidationError("A %s cannot be parent for a %s" % (
-            obj.__class__.__name__, child_model.__name__))
+        raise ValidationError(str("A %s cannot be parent for a %s" % (
+            obj.__class__.__name__, child_model.__name__)))
     attrs[parent_link_field.name] = obj
     # ~ for pm,pf in child_model._meta.parents.items(): # pm : parent model, pf : parent link field
         #~ attrs[pf.name] = obj
     #~ attrs["%s_ptr" % obj.__class__.__name__.lower()] = obj
-    for field in obj._meta.fields:
-        attrs[field.name] = getattr(obj, field.name)
-    #~ logger.info(u"Promote %s to %s : attrs=%s",
-        #~ obj.__class__.__name__,child_model.__name__,attrs)
-    #~ logger.debug(u"Promote %s to %s",
-        #~ obj.__class__.__name__,child_model.__name__)
+    if AFTER17:
+        fields_list = obj._meta.concrete_fields
+    else:
+        fields_list = obj._meta.fields
+    for field in fields_list:
+        if not DJANGO2 or DJANGO2 and not field.primary_key:
+            attrs[field.name] = getattr(obj, field.name)
     new_obj = child_model(**attrs)
     #~ logger.info("20120830 insert_child %s",obj2str(new_obj))
+
     new_obj.save()
+    # on_add_child.send(sender=obj, child=new_obj)
+    if full_clean:
+        try:
+            new_obj.full_clean()
+            new_obj.save()
+        except ValidationError as e:
+            msg = obj.error2str(e)
+            raise ValidationError(
+                _("Problem while inserting %(child)s "
+                  "child of %(parent)s: %(message)s") %
+                dict(child=child_model.__name__,
+                     parent=obj.__class__.__name__,
+                     message=msg))
     return new_obj
+        
+
+def mtichild(p, model, **kw):
+    """Create an MTI child, optionally set some attributes, save it to the
+    database and then return the new database object.
+
+    """
+    c = insert_child(p, model)
+    for k, v in kw.items():
+        setattr(c, k, v)
+    c.full_clean()
+    c.save()
+    return model.objects.get(pk=p.pk)
 
 #~ def insert_child_and_save(obj,child_model,**attrs):
     #~ """
@@ -206,24 +219,27 @@ def insert_child(obj, child_model, **attrs):
 
 
 class EnableChild(VirtualField):
+    """Rendered as a checkbox that indicates whether an mti child of the
+    given model exists.
+
+    Deprecated. Use polymorphic.Polymorphic instead.
 
-    """
-    Rendered as a checkbox that indicates whether an mti 
-    child of the given model exists.
     """
 
     editable = True
     #~ default = models.NOT_PROVIDED
 
     def __init__(self, child_model, **kw):
+        raise Exception("No longer recommended. "
+                        "Use `lino.mixins.polymorphic` instead.")
         kw.update(default=False)
         self.child_model = child_model
         VirtualField.__init__(self, models.BooleanField(**kw), self.has_child)
 
     def is_enabled(self, lh):
-        """
-        When a FormLayout is inherited by an MTI 
-        child, EnableChild fields must be disabled.
+        """When a FormLayout is inherited by an MTI child, EnableChild fields
+        must be disabled.
+
         """
         return lh.layout._datasource.model != self.child_model \
             and issubclass(self.child_model, lh.layout._datasource.model)
@@ -234,10 +250,10 @@ class EnableChild(VirtualField):
         VirtualField.attach_to_model(self, model, name)
 
     def has_child(self, obj, request=None):
-        """
-        Returns True if `obj` has an MTI child in `self.child_model`.
-        The optional 2nd argument `request` (passed from
+        """Returns True if `obj` has an MTI child in `self.child_model`.  The
+        optional 2nd argument `request` (passed from
         `VirtualField.value_from_object`) is ignored.
+
         """
         try:
             getattr(obj, self.child_model.__name__.lower())
@@ -257,8 +273,8 @@ class EnableChild(VirtualField):
             if not v:
                 if ar is not None:
                     pre_remove_child.send(
-                        sender=obj, request=ar.request, child=self.child_model)
-                    #~ changes.log_remove_child(ar.request,obj,self.child_model)
+                        sender=obj, request=ar.request,
+                        child=self.child_model)
                 delete_child(obj, self.child_model, ar)
         else:
             #~ logger.debug('set_value_in_object : %s has no child %s',
@@ -267,9 +283,10 @@ class EnableChild(VirtualField):
                 # child doesn't exist. insert if it should
                 if ar is not None:
                     pre_add_child.send(
-                        sender=obj, request=ar.request, child=self.child_model)
-                    #~ changes.log_add_child(ar.request,obj,self.child_model)
-                insert_child(obj, self.child_model)
+                        sender=obj, request=ar.request,
+                        child=self.child_model)
+                insert_child(obj, self.child_model, full_clean=True)
 
 
 from lino.utils.dpy import create_mti_child as create_child
+
