@@ -7,6 +7,7 @@ instructed there).
 :func:`latex_body` deduced from :func:`html_body`.
 
 """
+from __future__ import print_function
 
 #~ import traceback
 from docutils import core, io, nodes, utils
@@ -149,6 +150,8 @@ class HTMLTranslator(html4css1.HTMLTranslator):
 
 class Writer(html4css1.Writer):
 
+    # settings_default_overrides = dict(newlines=False)
+
     def __init__(self):
         writers.Writer.__init__(self)
         self.translator_class = HTMLTranslator
@@ -227,7 +230,7 @@ def restify(input_string, source_path=None, destination_path=None,
     #~ if output_encoding != 'unicode':
         #~ fragment = fragment.encode(output_encoding)
     #~ print __file__, repr(fragment)
-    return fragment
+    return fragment  #.decode('utf8')
 
 
 def rst2odt(input_string, source_path=None, destination_path=None,
@@ -273,8 +276,8 @@ def rst2odt(input_string, source_path=None, destination_path=None,
         writer=MyWriter(),
         reader=Reader(),
         settings_overrides=overrides)
-    print 20120311, parts.keys()
-    print 20120311, parts['content']
+    print(20120311, list(parts.keys()))
+    print(20120311, parts['content'])
     raise Exception("20120311")
     fragment = parts['whole']
     fragment = parts['html_body']
@@ -405,6 +408,6 @@ A table:
 
 
 """
-    print rst2odt(test)
+    print(rst2odt(test))
     #~ print restify(test)
     #~ print latex_body(test)
